@@ -1069,7 +1069,8 @@ static short translevel;
 static bool badtrans;
 
 /* Through globals, make error info available to the application. */
-int rv_lastStatus, rv_vendorStatus, rv_stmtOffset;
+int rv_lastStatus, rv_stmtOffset;
+long rv_vendorStatus;
 char *rv_badToken;
 
 static void debugStatement(void)
@@ -1102,63 +1103,8 @@ void sql_exception(int errnum)
 	exclist = list;
 } /* sql_exception */
 
-/* text descriptions corresponding to our generic SQL error codes */
-static char *sqlErrorList[] = {0,
-	"miscelaneous SQL error",
-	"syntax error in SQL statement",
-	"filename cannot be used by SQL",
-	"cannot convert/compare the columns/constants in the SQL statement",
-	"bad string subscripting",
-	"bad use of the rowid construct",
-	"bad use of a blob column",
-	"bad use of aggregate operators or columns",
-	"bad use of a view",
-	"bad use of a serial column",
-	"bad use of a temp table",
-	"operation cannot cross databases",
-	"database is fucked up",
-	"query interrupted by user",
-	"could not connect to the database",
-	"database has not yet been selected",
-	"table not found",
-	"duplicate table",
-	"ambiguous table",
-	"column not found",
-	"duplicate column",
-	"ambiguous column",
-	"index not found",
-	"duplicate index",
-	"constraint not found",
-	"duplicate constraint",
-	"stored procedure not found",
-	"duplicate stored procedure",
-	"synonym not found",
-	"duplicate synonym",
-	"table has no primary or unique key",
-	"duplicate primary or unique key",
-	"cursor not specified, or cursor is not available",
-	"duplicate cursor",
-	"the database lacks the resources needed to complete this query",
-	"check constrain violated",
-	"referential integrity violated",
-	"cannot manage or complete the transaction",
-	"long transaction, too much log data generated",
-	"this operation must be run inside a transaction",
-	"cannot open, read, write, close, or otherwise manage a blob",
-	"row, table, page, or database is already locked, or cannot be locked",
-	"inserting null into a not null column",
-	"no permission to modify the database in this way",
-	"no current row established",
-	"many rows were found where one was expected",
-	"cannot union these select statements together",
-	"cannot access or write the audit trail",
-	"could not run SQL or gather data from a remote host",
-	"where clause is semantically unmanageable",
-	"deadlock detected",
-0};
-
-/* map Informix errors to our own exception codes, as defined in c_sql.h. */
-static struct ERRORMAP {
+/* map Informix errors to our own exception codes, as defined in dbapi.h. */
+static const struct ERRORMAP {
 	short infcode;
 	short excno;
 } errormap[] = {
@@ -1522,7 +1468,7 @@ static struct ERRORMAP {
 
 static int errTranslate(int code)
 {
-	struct ERRORMAP *e;
+	const struct ERRORMAP *e;
 
 	for(e=errormap; e->infcode; ++e) {
 		if(e->infcode == code)
@@ -1535,11 +1481,14 @@ static bool errorTrap(void)
 {
 short i;
 
-rv_lastStatus = rv_vendorStatus = 0; /* innocent until proven guilty */
+/* innocent until proven guilty */
+rv_lastStatus = 0;
+rv_vendorStatus = 0;
 rv_stmtOffset = 0;
 rv_badToken = 0;
 if(ENGINE_ERRCODE >= 0) return false; /* no problem */
 
+        /* log the SQL statement that elicitted the error */
 showStatement();
 rv_vendorStatus = -ENGINE_ERRCODE;
 rv_lastStatus = errTranslate(rv_vendorStatus);
@@ -1548,15 +1497,14 @@ rv_badToken = sqlca.sqlerrm;
 if(!rv_badToken[0]) rv_badToken = 0;
 
 /* if the application didn't trap for this exception, blow up! */
-if(exclist) {
-for(i=0; exclist[i]; ++i) {
+if(exclist)
+for(i=0; exclist[i]; ++i)
 if(exclist[i] == rv_lastStatus) {
 exclist = 0; /* we've spent that exception */
 return true;
 }
-}
-}
 
+/* Remember, errorPrint() should not return. */
 errorPrint("2SQL error %d, %s", rv_vendorStatus, sqlErrorList[rv_lastStatus]);
 return true; /* make the compiler happy */
 } /* errorTrap */
@@ -1574,11 +1522,9 @@ char cname[8]; /* cursor name */
 struct sqlda *desc;
 char rv_type[NUMRETS];
 long rownum;
-short alloc;
 short cid; /* cursor ID */
 char flag;
 char numRets;
-char **fl; /* array of fetched lines */
 } ocurs[NUMCURSORS];
 
 /* values for struct OCURS.flag */
@@ -1617,6 +1563,7 @@ memcpy(rv_type, o->rv_type, NUMRETS);
 return o;
 } /* findCursor */
 
+/* This doesn't close/free anything; it simply puts variables in an initial state. */
 /* part of the disconnect() procedure */
 static void clearAllCursors(void)
 {
@@ -1627,12 +1574,6 @@ static void clearAllCursors(void)
 		if(o->flag == CURSOR_NONE) continue;
 		o->flag = CURSOR_NONE;
 o->rownum = 0;
-		if(o->fl) {
-			for(j=0; j<o->alloc; ++j)
-				nzFree(o->fl[j]);
-			nzFree(o->fl);
-			o->fl = 0;
-		}
 	} /* loop over cursors */
 
 	translevel = 0;
@@ -1649,7 +1590,7 @@ void sql_connect(const char *db, const char *login, const char *pw)
 /*
  * $char *dblocal = (char*)db;
  */
-#line 637 "dbinfx.ec"
+#line 578 "dbinfx.ec"
   char *dblocal = (char*)db;
 login = pw = 0; /* not used here, so make the compiler happy */
 if(isnullstring(dblocal)) {
@@ -1664,11 +1605,11 @@ if(sql_database) {
 /*
  * $disconnect current;
  */
-#line 648 "dbinfx.ec"
+#line 589 "dbinfx.ec"
   {
-#line 648 "dbinfx.ec"
+#line 589 "dbinfx.ec"
   sqli_connect_close(3, (char *)0, 0, 0);
-#line 648 "dbinfx.ec"
+#line 589 "dbinfx.ec"
   }
 clearAllCursors();
 sql_database = 0;
@@ -1679,11 +1620,11 @@ sql_database = 0;
 /*
  * $connect to :dblocal;
  */
-#line 655 "dbinfx.ec"
+#line 596 "dbinfx.ec"
   {
-#line 655 "dbinfx.ec"
+#line 596 "dbinfx.ec"
   sqli_connect_open(ESQLINTVERSION, 0, dblocal, (char *)0, (ifx_conn_t *)0, 0);
-#line 655 "dbinfx.ec"
+#line 596 "dbinfx.ec"
   }
 if(errorTrap()) return;
 sql_database = dblocal;
@@ -1694,21 +1635,21 @@ debugStatement();
 /*
  * $ set lock mode to wait;
  */
-#line 662 "dbinfx.ec"
+#line 603 "dbinfx.ec"
   {
-#line 662 "dbinfx.ec"
+#line 603 "dbinfx.ec"
   static const char *sqlcmdtxt[] =
-#line 662 "dbinfx.ec"
+#line 603 "dbinfx.ec"
     {
-#line 662 "dbinfx.ec"
+#line 603 "dbinfx.ec"
     "set lock mode to wait",
     0
     };
-#line 662 "dbinfx.ec"
+#line 603 "dbinfx.ec"
   static ifx_statement_t _SQ0 = {0};
-#line 662 "dbinfx.ec"
+#line 603 "dbinfx.ec"
   sqli_stmt(ESQLINTVERSION, &_SQ0, sqlcmdtxt, 0, (ifx_sqlvar_t *)0, (struct value *)0, (ifx_literal_t *)0, (ifx_namelist_t *)0, (ifx_cursor_t *)0, -1, 0, 0);
-#line 662 "dbinfx.ec"
+#line 603 "dbinfx.ec"
   }
 if(errorTrap()) {
 abort:
@@ -1718,21 +1659,21 @@ return;
 /*
  * $ set isolation to committed read;
  */
-#line 668 "dbinfx.ec"
+#line 609 "dbinfx.ec"
   {
-#line 668 "dbinfx.ec"
+#line 609 "dbinfx.ec"
   static const char *sqlcmdtxt[] =
-#line 668 "dbinfx.ec"
+#line 609 "dbinfx.ec"
     {
-#line 668 "dbinfx.ec"
+#line 609 "dbinfx.ec"
     "set isolation to committed read",
     0
     };
-#line 668 "dbinfx.ec"
+#line 609 "dbinfx.ec"
   static ifx_statement_t _SQ0 = {0};
-#line 668 "dbinfx.ec"
+#line 609 "dbinfx.ec"
   sqli_stmt(ESQLINTVERSION, &_SQ0, sqlcmdtxt, 0, (ifx_sqlvar_t *)0, (struct value *)0, (ifx_literal_t *)0, (ifx_namelist_t *)0, (ifx_cursor_t *)0, -1, 0, 0);
-#line 668 "dbinfx.ec"
+#line 609 "dbinfx.ec"
   }
 if(errorTrap()) goto abort;
 exclist = 0;
@@ -1746,11 +1687,11 @@ if(sql_database) {
 /*
  * $disconnect current;
  */
-#line 678 "dbinfx.ec"
+#line 619 "dbinfx.ec"
   {
-#line 678 "dbinfx.ec"
+#line 619 "dbinfx.ec"
   sqli_connect_close(3, (char *)0, 0, 0);
-#line 678 "dbinfx.ec"
+#line 619 "dbinfx.ec"
   }
 clearAllCursors();
 sql_database = 0;
@@ -1781,19 +1722,21 @@ void sql_begTrans(void)
 {
 	rv_lastStatus = 0;
 	checkConnect();
-		stmt_text = "begin work";
-		debugStatement();
+stmt_text = 0;
+
 	/* count the nesting level of transactions. */
 	if(!translevel) {
 		badtrans = false;
+		stmt_text = "begin work";
+		debugStatement();
 /*
  * 		$begin work;
  */
-#line 713 "dbinfx.ec"
+#line 656 "dbinfx.ec"
   {
-#line 713 "dbinfx.ec"
+#line 656 "dbinfx.ec"
   sqli_trans_begin2((mint)1);
-#line 713 "dbinfx.ec"
+#line 656 "dbinfx.ec"
   }
 		if(errorTrap()) return;
 	}
@@ -1806,6 +1749,7 @@ static void endTrans(bool commit)
 {
 	rv_lastStatus = 0;
 	checkConnect();
+stmt_text = 0;
 
 	if(translevel == 0)
 		errorPrint("2end transaction without a matching begTrans()");
@@ -1820,11 +1764,11 @@ static void endTrans(bool commit)
 /*
  * 			$commit work;
  */
-#line 736 "dbinfx.ec"
+#line 680 "dbinfx.ec"
   {
-#line 736 "dbinfx.ec"
+#line 680 "dbinfx.ec"
   sqli_trans_commit();
-#line 736 "dbinfx.ec"
+#line 680 "dbinfx.ec"
   }
 			if(ENGINE_ERRCODE) ++translevel;
 			errorTrap();
@@ -1837,11 +1781,11 @@ static void endTrans(bool commit)
 /*
  * 			$rollback work;
  */
-#line 745 "dbinfx.ec"
+#line 689 "dbinfx.ec"
   {
-#line 745 "dbinfx.ec"
+#line 689 "dbinfx.ec"
   sqli_trans_rollback();
-#line 745 "dbinfx.ec"
+#line 689 "dbinfx.ec"
   }
 			if(ENGINE_ERRCODE) --translevel;
 			errorTrap();
@@ -1869,21 +1813,21 @@ void sql_deferConstraints(void)
 /*
  * 	$set constraints all deferred;
  */
-#line 769 "dbinfx.ec"
+#line 713 "dbinfx.ec"
   {
-#line 769 "dbinfx.ec"
+#line 713 "dbinfx.ec"
   static const char *sqlcmdtxt[] =
-#line 769 "dbinfx.ec"
+#line 713 "dbinfx.ec"
     {
-#line 769 "dbinfx.ec"
+#line 713 "dbinfx.ec"
     "set constraints all deferred",
     0
     };
-#line 769 "dbinfx.ec"
+#line 713 "dbinfx.ec"
   static ifx_statement_t _SQ0 = {0};
-#line 769 "dbinfx.ec"
+#line 713 "dbinfx.ec"
   sqli_stmt(ESQLINTVERSION, &_SQ0, sqlcmdtxt, 0, (ifx_sqlvar_t *)0, (struct value *)0, (ifx_literal_t *)0, (ifx_namelist_t *)0, (ifx_cursor_t *)0, -1, 0, 0);
-#line 769 "dbinfx.ec"
+#line 713 "dbinfx.ec"
   }
 	errorTrap();
 	exclist = 0;
@@ -1906,11 +1850,6 @@ Future versions of our line formatting software may support a %blob directive,
 which makes sense only when the formatted string is destined for SQL.
 *********************************************************************/
 
-/* information about the blob being fetched */
-const char *rv_blobFile;
-bool rv_blobAppend;
-void *rv_blobLoc; /* location of blob in memory */
-int rv_blobSize; /* size of blob in bytes */
 static loc_t blobstruct; /* Informix structure to manage the blob */
 
 /* insert a blob into the database */
@@ -1920,12 +1859,12 @@ const char *filename, void *offset, int length)
 /*
  * $char blobcmd[100];
  */
-#line 802 "dbinfx.ec"
+#line 741 "dbinfx.ec"
   char blobcmd[100];
 /*
  * $loc_t insblob;
  */
-#line 803 "dbinfx.ec"
+#line 742 "dbinfx.ec"
   loc_t insblob;
 
 /* basic sanity checks */
@@ -1964,29 +1903,29 @@ debugStatement();
 /*
  * $prepare blobinsert from :blobcmd;
  */
-#line 838 "dbinfx.ec"
+#line 777 "dbinfx.ec"
   {
-#line 838 "dbinfx.ec"
+#line 777 "dbinfx.ec"
   sqli_prep(ESQLINTVERSION, _Cn1, blobcmd,(ifx_literal_t *)0, (ifx_namelist_t *)0, -1, 0, 0 ); 
-#line 838 "dbinfx.ec"
+#line 777 "dbinfx.ec"
   }
 if(errorTrap()) return;
 /*
  * $execute blobinsert using :insblob;
  */
-#line 840 "dbinfx.ec"
+#line 779 "dbinfx.ec"
   {
-#line 840 "dbinfx.ec"
+#line 779 "dbinfx.ec"
   static ifx_sqlvar_t _sqibind[] = 
     {
       { 113, sizeof(insblob), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
-#line 840 "dbinfx.ec"
+#line 779 "dbinfx.ec"
     };
   static ifx_sqlda_t _SD0 = { 1, _sqibind, {0}, 1, 0 };
-#line 840 "dbinfx.ec"
+#line 779 "dbinfx.ec"
   _sqibind[0].sqldata = (char *) &insblob;
   sqli_exec(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, _Cn1, 769), &_SD0, (char *)0, (struct value *)0, (ifx_sqlda_t *)0, (char *)0, (struct value *)0, 0);
-#line 840 "dbinfx.ec"
+#line 779 "dbinfx.ec"
   }
 errorTrap();
 rv_lastNrows = sqlca.sqlerrd[2];
@@ -2011,12 +1950,6 @@ into our own 4-byte representations.
 The same for time intervals, money, etc.
 *********************************************************************/
 
-/* Arrays that hold the return values from a select statement. */
-int rv_numRets; /* number of returned values */
-char rv_type[NUMRETS+1]; /* datatypes of returned values */
-char rv_name[NUMRETS+1][COLNAMELEN]; /* column names */
-LF  rv_data[NUMRETS]; /* the returned values */
-int rv_lastNrows, rv_lastRowid, rv_lastSerial;
 /* Temp area to read the Informix values, as strings */
 static char retstring[NUMRETS][STRINGLEN+4];
 static va_list sqlargs;
@@ -2223,17 +2156,6 @@ sprintf(q, "%ld.%02d", l/100, l%100);
 if(!first) va_end(sqlargs);
 } /* retsCopy */
 
-/* convert column name into column index */
-int findcol_byname(const char *name)
-{
-	int i;
-	for(i=0; rv_name[i][0]; ++i)
-		if(stringEqual(name, rv_name[i])) break;
-	if(!rv_name[i][0])
-		errorPrint("2Column %s not found in the columns or aliases of your select statement", name);
-	return i;
-} /* findcol_byname */
-
 /* make sure we got one return value, and it is integer compatible */
 static long oneRetValue(void)
 {
@@ -2261,12 +2183,12 @@ static struct sqlda *prepare(const char *stmt_parm, const char *sname_parm)
 /*
  * $char*stmt = (char*)stmt_parm;
  */
-#line 1111 "dbinfx.ec"
+#line 1033 "dbinfx.ec"
   char *stmt = (char*)stmt_parm;
 /*
  * $char*sname = (char*)sname_parm;
  */
-#line 1112 "dbinfx.ec"
+#line 1034 "dbinfx.ec"
   char *sname = (char*)sname_parm;
 struct sqlda *desc;
 struct sqlvar_struct   *v;
@@ -2295,11 +2217,11 @@ rv_lastNrows = rv_lastRowid = rv_lastSerial = 0;
 /*
  * $prepare :sname from :stmt;
  */
-#line 1137 "dbinfx.ec"
+#line 1059 "dbinfx.ec"
   {
-#line 1137 "dbinfx.ec"
+#line 1059 "dbinfx.ec"
   sqli_prep(ESQLINTVERSION, sname, stmt,(ifx_literal_t *)0, (ifx_namelist_t *)0, -1, 0, 0 ); 
-#line 1137 "dbinfx.ec"
+#line 1059 "dbinfx.ec"
   }
 if(errorTrap()) return 0;
 
@@ -2307,11 +2229,11 @@ if(errorTrap()) return 0;
 /*
  * $describe: sname into desc;
  */
-#line 1141 "dbinfx.ec"
+#line 1063 "dbinfx.ec"
   {
-#line 1141 "dbinfx.ec"
+#line 1063 "dbinfx.ec"
   sqli_describe_stmt(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, sname, 257), &desc, 0);
-#line 1141 "dbinfx.ec"
+#line 1063 "dbinfx.ec"
   }
 if(!desc) errorPrint("2$describe couldn't allocate descriptor");
 rv_numRets = desc->sqld;
@@ -2398,12 +2320,12 @@ struct sqlda *desc;
 /*
  * $static char singlestatement[] = "single_use_stmt";
  */
-#line 1224 "dbinfx.ec"
+#line 1146 "dbinfx.ec"
 static   char singlestatement[] = "single_use_stmt";
 /*
  * $static char singlecursor[] = "single_use_cursor";
  */
-#line 1225 "dbinfx.ec"
+#line 1147 "dbinfx.ec"
 static   char singlecursor[] = "single_use_cursor";
 int i;
 bool notfound = false;
@@ -2420,11 +2342,11 @@ errorPrint("2SQL select statement returns no values");
 /*
  * $execute :singlestatement;
  */
-#line 1238 "dbinfx.ec"
+#line 1160 "dbinfx.ec"
   {
-#line 1238 "dbinfx.ec"
+#line 1160 "dbinfx.ec"
   sqli_exec(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, singlestatement, 257), (ifx_sqlda_t *)0, (char *)0, (struct value *)0, (ifx_sqlda_t *)0, (char *)0, (struct value *)0, 0);
-#line 1238 "dbinfx.ec"
+#line 1160 "dbinfx.ec"
   }
 notfound = true;
 } else { /* end no return values */
@@ -2436,11 +2358,11 @@ errorPrint("2SQL statement returns %d values", rv_numRets);
 /*
  * $execute: singlestatement into descriptor desc;
  */
-#line 1246 "dbinfx.ec"
+#line 1168 "dbinfx.ec"
   {
-#line 1246 "dbinfx.ec"
+#line 1168 "dbinfx.ec"
   sqli_exec(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, singlestatement, 257), (ifx_sqlda_t *)0, (char *)0, (struct value *)0, desc, (char *)0, (struct value *)0, 0);
-#line 1246 "dbinfx.ec"
+#line 1168 "dbinfx.ec"
   }
 }
 
@@ -2459,11 +2381,11 @@ rv_lastSerial = sqlca.sqlerrd[1];
 /*
  * $free :singlestatement;
  */
-#line 1261 "dbinfx.ec"
+#line 1183 "dbinfx.ec"
   {
-#line 1261 "dbinfx.ec"
+#line 1183 "dbinfx.ec"
   sqli_curs_free(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, singlestatement, 258));
-#line 1261 "dbinfx.ec"
+#line 1183 "dbinfx.ec"
   }
 errorTrap();
 nzFree(desc);
@@ -2487,18 +2409,20 @@ Run individual select or execute statements, using the above internal routine.
 
 /* pointer to vararg list; most of these are vararg functions */
 /* execute a stand-alone statement with no % formatting of the string */
-void sql_execNF(const char *stmt)
+bool sql_execNF(const char *stmt)
 {
-	execInternal(stmt, 1);
+	return execInternal(stmt, 1);
 } /* sql_execNF */
 
 /* execute a stand-alone statement with % formatting */
-void sql_exec(const char *stmt, ...)
+bool sql_exec(const char *stmt, ...)
 {
+bool ok;
 	va_start(sqlargs, stmt);
 	stmt = lineFormatStack(stmt, 0, &sqlargs);
-	execInternal(stmt, 1);
+	ok = execInternal(stmt, 1);
 	va_end(sqlargs);
+return ok;
 } /* sql_exec */
 
 /* run a select statement with no % formatting of the string */
@@ -2580,7 +2504,7 @@ static int prepareCursor(const char *stmt, bool scrollflag)
 /*
  * $char *internal_sname, *internal_cname;
  */
-#line 1374 "dbinfx.ec"
+#line 1298 "dbinfx.ec"
   char *internal_sname, *internal_cname;
 struct OCURS *o = findNewCursor();
 
@@ -2601,21 +2525,21 @@ if(scrollflag)
 /*
  * $declare :internal_cname scroll cursor with hold for :internal_sname;
  */
-#line 1391 "dbinfx.ec"
+#line 1315 "dbinfx.ec"
   {
-#line 1391 "dbinfx.ec"
+#line 1315 "dbinfx.ec"
   sqli_curs_decl_dynm(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, internal_cname, 0), internal_cname, sqli_curs_locate(ESQLINTVERSION, internal_sname, 1), 4128, 0);
-#line 1391 "dbinfx.ec"
+#line 1315 "dbinfx.ec"
   }
 else
 /*
  * $declare :internal_cname cursor with hold for :internal_sname;
  */
-#line 1393 "dbinfx.ec"
+#line 1317 "dbinfx.ec"
   {
-#line 1393 "dbinfx.ec"
+#line 1317 "dbinfx.ec"
   sqli_curs_decl_dynm(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, internal_cname, 0), internal_cname, sqli_curs_locate(ESQLINTVERSION, internal_sname, 1), 4096, 0);
-#line 1393 "dbinfx.ec"
+#line 1317 "dbinfx.ec"
   }
 if(errorTrap()) {
 nzFree(o->desc);
@@ -2625,7 +2549,6 @@ return -1;
 o->numRets = rv_numRets;
 memcpy(o->rv_type, rv_type, NUMRETS);
 o->flag = CURSOR_PREPARED;
-o->fl = 0; /* just to make sure */
 return o->cid;
 } /* prepareCursor */
 
@@ -2653,7 +2576,7 @@ short i;
 /*
  * $char *internal_sname, *internal_cname;
  */
-#line 1427 "dbinfx.ec"
+#line 1350 "dbinfx.ec"
   char *internal_sname, *internal_cname;
 struct OCURS *o = findCursor(cid);
 if(o->flag == CURSOR_OPENED)
@@ -2664,19 +2587,14 @@ debugExtra("open");
 /*
  * $open :internal_cname;
  */
-#line 1434 "dbinfx.ec"
+#line 1357 "dbinfx.ec"
   {
-#line 1434 "dbinfx.ec"
+#line 1357 "dbinfx.ec"
   sqli_curs_open(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, internal_cname, 256), (ifx_sqlda_t *)0, (char *)0, (struct value *)0, 0, 0);
-#line 1434 "dbinfx.ec"
+#line 1357 "dbinfx.ec"
   }
 if(!errorTrap()) o->flag = CURSOR_OPENED;
 o->rownum = 0;
-if(o->fl)
-for(i=0; i<o->alloc; ++i) {
-nzFree(o->fl[i]);
-o->fl[i] = 0;
-}
 exclist = 0;
 } /* sql_open */
 
@@ -2703,7 +2621,7 @@ void sql_close(int cid)
 /*
  * $char *internal_sname, *internal_cname;
  */
-#line 1465 "dbinfx.ec"
+#line 1383 "dbinfx.ec"
   char *internal_sname, *internal_cname;
 struct OCURS *o = findCursor(cid);
 if(o->flag < CURSOR_OPENED)
@@ -2713,23 +2631,24 @@ debugExtra("close");
 /*
  * $close :internal_cname;
  */
-#line 1471 "dbinfx.ec"
+#line 1389 "dbinfx.ec"
   {
-#line 1471 "dbinfx.ec"
+#line 1389 "dbinfx.ec"
   sqli_curs_close(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, internal_cname, 256));
-#line 1471 "dbinfx.ec"
+#line 1389 "dbinfx.ec"
   }
 if(errorTrap()) return;
 o->flag = CURSOR_PREPARED;
 exclist = 0;
 } /* sql_close */
 
-void sql_free( int cid)
+void
+sql_free( int cid)
 {
 /*
  * $char *internal_sname, *internal_cname;
  */
-#line 1479 "dbinfx.ec"
+#line 1398 "dbinfx.ec"
   char *internal_sname, *internal_cname;
 struct OCURS *o = findCursor(cid);
 if(o->flag == CURSOR_OPENED)
@@ -2739,11 +2658,11 @@ debugExtra("free");
 /*
  * $free :internal_sname;
  */
-#line 1485 "dbinfx.ec"
+#line 1404 "dbinfx.ec"
   {
-#line 1485 "dbinfx.ec"
+#line 1404 "dbinfx.ec"
   sqli_curs_free(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, internal_sname, 258));
-#line 1485 "dbinfx.ec"
+#line 1404 "dbinfx.ec"
   }
 if(errorTrap()) return;
 o->flag = CURSOR_NONE;
@@ -2751,14 +2670,6 @@ nzFree(o->desc);
 rv_numRets = 0;
 memset(rv_name, 0, sizeof(rv_name));
 memset(rv_type, 0, sizeof(rv_type));
-if(o->fl) { /* free any cached lines */
-short i;
-for(i=0; i<o->alloc; ++i)
-nzFree(o->fl[i]);
-nzFree(o->fl);
-o->fl = 0;
-o->alloc = 0;
-}
 exclist = 0;
 } /* sql_free */
 
@@ -2774,17 +2685,18 @@ sql_free(cid);
 
 /* fetch row n from the open cursor.
  * Flag can be used to fetch first, last, next, or previous. */
-bool fetchInternal(int cid, long n, int flag, bool remember)
+static bool
+fetchInternal(int cid, long n, int flag, bool remember)
 {
 /*
  * $char *internal_sname, *internal_cname;
  */
-#line 1517 "dbinfx.ec"
+#line 1429 "dbinfx.ec"
   char *internal_sname, *internal_cname;
 /*
  * $long nextrow, lastrow;
  */
-#line 1518 "dbinfx.ec"
+#line 1430 "dbinfx.ec"
 long nextrow, lastrow;
 struct sqlda *internal_desc;
 struct OCURS *o = findCursor(cid);
@@ -2812,28 +2724,10 @@ if(flag == 2 && nextrow) --nextrow;
 }
 if(flag == 4) { /* fetch the last row */
 nextrow = nullint; /* we just lost track */
-if(o->fl && o->flag == CURSOR_PREPARED) {
-/* I'll assume you've read in all the rows, cursor is closed */
-for(nextrow=o->alloc-1; nextrow>=0; --nextrow)
-if(o->fl[nextrow]) break;
-++nextrow;
-}
 }
 
 if(!nextrow) goto fetchZero;
 
-/* see if we have cached this row */
-if(isnotnull(nextrow) && o->fl &&
-nextrow <= o->alloc && o->fl[nextrow-1]) {
-sql_mkload(o->fl[nextrow-1], '\177');
-/* don't run retsCleanup() here */
-rv_blobLoc = 0;
-rv_blobSize = nullint;
-o->rownum = nextrow;
-exclist = 0;
-return true;
-} /* bringing row out of cache */
-
 if(o->flag != CURSOR_OPENED)
 errorPrint("2cannot fetch from cursor %d, not yet opened", cid);
 
@@ -2859,48 +2753,48 @@ case 1:
 /*
  * $fetch :internal_cname using descriptor internal_desc;
  */
-#line 1589 "dbinfx.ec"
+#line 1483 "dbinfx.ec"
   {
-#line 1589 "dbinfx.ec"
+#line 1483 "dbinfx.ec"
   static _FetchSpec _FS0 = { 0, 1, 0 };
   sqli_curs_fetch(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, internal_cname, 256), (ifx_sqlda_t *)0, internal_desc, (char *)0, &_FS0);
-#line 1589 "dbinfx.ec"
+#line 1483 "dbinfx.ec"
   }
 break;
 case 2:
 /*
  * $fetch previous :internal_cname using descriptor internal_desc;
  */
-#line 1592 "dbinfx.ec"
+#line 1486 "dbinfx.ec"
   {
-#line 1592 "dbinfx.ec"
+#line 1486 "dbinfx.ec"
   static _FetchSpec _FS0 = { 0, 2, 0 };
   sqli_curs_fetch(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, internal_cname, 256), (ifx_sqlda_t *)0, internal_desc, (char *)0, &_FS0);
-#line 1592 "dbinfx.ec"
+#line 1486 "dbinfx.ec"
   }
 break;
 case 3:
 /*
  * $fetch first :internal_cname using descriptor internal_desc;
  */
-#line 1595 "dbinfx.ec"
+#line 1489 "dbinfx.ec"
   {
-#line 1595 "dbinfx.ec"
+#line 1489 "dbinfx.ec"
   static _FetchSpec _FS0 = { 0, 3, 0 };
   sqli_curs_fetch(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, internal_cname, 256), (ifx_sqlda_t *)0, internal_desc, (char *)0, &_FS0);
-#line 1595 "dbinfx.ec"
+#line 1489 "dbinfx.ec"
   }
 break;
 case 4:
 /*
  * $fetch last :internal_cname using descriptor internal_desc;
  */
-#line 1598 "dbinfx.ec"
+#line 1492 "dbinfx.ec"
   {
-#line 1598 "dbinfx.ec"
+#line 1492 "dbinfx.ec"
   static _FetchSpec _FS0 = { 0, 4, 0 };
   sqli_curs_fetch(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, internal_cname, 256), (ifx_sqlda_t *)0, internal_desc, (char *)0, &_FS0);
-#line 1598 "dbinfx.ec"
+#line 1492 "dbinfx.ec"
   }
 break;
 case 6:
@@ -2909,20 +2803,20 @@ errorPrint("2sql fetches absolute row using null index");
 /*
  * $fetch absolute :nextrow :internal_cname using descriptor internal_desc;
  */
-#line 1603 "dbinfx.ec"
+#line 1497 "dbinfx.ec"
   {
-#line 1603 "dbinfx.ec"
+#line 1497 "dbinfx.ec"
   static ifx_sqlvar_t _sqibind[] = 
     {
       { 103, sizeof(nextrow), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
-#line 1603 "dbinfx.ec"
+#line 1497 "dbinfx.ec"
     };
   static ifx_sqlda_t _SD0 = { 1, _sqibind, {0}, 1, 0 };
   static _FetchSpec _FS1 = { 0, 6, 0 };
-#line 1603 "dbinfx.ec"
+#line 1497 "dbinfx.ec"
   _sqibind[0].sqldata = (char *) &nextrow;
   sqli_curs_fetch(ESQLINTVERSION, sqli_curs_locate(ESQLINTVERSION, internal_cname, 256), &_SD0, internal_desc, (char *)0, &_FS1);
-#line 1603 "dbinfx.ec"
+#line 1497 "dbinfx.ec"
   }
 break;
 default:
@@ -2935,9 +2829,6 @@ exclist = 0;
 if(ENGINE_ERRCODE == 100) return false; /* not found */
 o->rownum = nextrow;
 
-/* remember the unload image of this line */
-if(remember)
-sql_cursorUpdLine(cid, cloneString(sql_mkunld('\177')));
 return true;
 } /* fetchInternal */
 
@@ -2987,328 +2878,46 @@ bool sql_fetchAbs(int cid, long rownum, ...)
 } /* sql_fetchAbs */
 
 
-/* the inverse of sql_mkunld() */
-void sql_mkload(const char *line, char delim)
-{
-char *s, *t;
-int data;
-short i;
-
-for(i = 0, s = (char*)line; *s; ++i, *t=delim, s = t+1) {
-t = strchr(s, delim);
-if(!t) errorPrint("2sql load line does not end in a delimiter");
-*t = 0;
-if(i >= rv_numRets)
-errorPrint("2sql load line contains more than %d fields", rv_numRets);
-
-switch(rv_type[i]) {
-case 'N':
-if(!*s) { data = nullint; break; }
-data = strtol(s, &s, 10);
-if(*s) errorPrint("2sql load, cannot convert string to integer");
-break;
-
-case 'S':
-if((unsigned)strlen(s) > STRINGLEN)
-errorPrint("2sql load line has a string that is too long");
-strcpy(retstring[i], s);
-data = (int) retstring[i];
-if(!*s) data = 0;
-break;
-
-case 'F':
-rv_data[i].f = *s ? atof(s) : nullfloat;
-continue;
-
-case 'D':
-data = stringDate(s,0);
-if(data == -1)
-errorPrint("2sql load, cannot convert string to date");
-break;
-
-case 'C':
-data = *s;
-if(data && s[1])
-errorPrint("2sql load, character field contains more than one character");
-break;
-
-case 'I':
-data = stringTime(s);
-if(data == -1)
-errorPrint("2sql load, cannot convert string to time interval");
-break;
-
-default:
-errorPrint("2sql load cannot convert into type %c", rv_type[i]);
-} /* switch on type */
-
-rv_data[i].l = data;
-} /* loop over fields in line */
-
-if(i != rv_numRets)
-errorPrint("2sql load line contains %d fields, %d expected", i, rv_numRets);
-} /* sql_mkload */
-
-
-/*********************************************************************
-We maintain our own cache of fetched lines.
-Why?  You might ask.
-After all, Informix already maintains a cache of fetched lines.
-That's what the open cursor is for.
-Looks like serious wheel reinvention to me.
-Perhaps, but you can't change the data in the cache that Informix maintains.
-This is something Powerbuild et al discovered over a decade ago.
-Consider a simple spreadsheet application.
-You update the value in one of the cells, thereby updating the row
-in the database.  Now scroll down to the next page, and then back again.
-If you fetch from the open cursor you will get the old data, before the
-change was made, even though the new data is safely ensconsed in the database.
-Granted one could reopen the cursor and fetch the new data,
-but this can be slow for certain queries (sometimes a couple minutes).
-In other words, rebuilding the cursor is not really an option.
-So we are forced to retain a copy of the data in our program and change it
-whenever we update the database.
-Unfortunately the following 3 routines were developed separately,
-and they are wildly inconsistent.  Some take a row number while
-others assume you are modifying the current row as stored in o->rownum.
-Some accept a line of tex, the unload image of the fetch data, while
-others build the line of text from the fetched data in rv_data[].
-I apologize for this confusion; clearly a redesign is called for.
-*********************************************************************/
-
-/* update the text of a fetched line,
- * so we get this same text again when we refetch the line later.
- * These text changes corespond to the database changes that form an update.
- * We assume the line has been allocated using malloc(). */
-void sql_cursorUpdLine(int cid, const char *line)
-{
-struct OCURS *o = findCursor(cid);
-int n = o->rownum-1;
-
-if(n >= CACHELIMIT)
-errorPrint("2SQL cursor caches too many lines, limit %d", CACHELIMIT);
-
-if(n >= o->alloc) {
-/* running off the end, allocate 128 at a time */
-short oldalloc = o->alloc;
-o->alloc = n + 128;
-if(!oldalloc)
-o->fl = (char **) allocMem(o->alloc*sizeof(char*));
-else
-o->fl = (char**) reallocMem((void*)o->fl, o->alloc*sizeof(char*));
-memset(o->fl+oldalloc, 0, (o->alloc-oldalloc)*sizeof(char*));
-} /* allocating more space */
-
-nzFree(o->fl[n]);
-o->fl[n] = (char*)line;
-} /* sql_cursorUpdLine */
-
-void sql_cursorDelLine(int cid, int rownum)
-{
-struct OCURS *o = findCursor(cid);
-o->rownum = rownum;
---rownum;
-if(rownum >= o->alloc || !o->fl[rownum])
-errorPrint("2cursorDelLine(%d)", rownum);
-nzFree(o->fl[rownum]);
-if(rownum < o->alloc-1)
-memcpy(o->fl+rownum, o->fl+rownum+1, (o->alloc-rownum-1)*sizeof(char *));
-o->fl[o->alloc-1] = 0;
-/* back up the row number if we deleted the last row */
-if(!o->fl[rownum]) --o->rownum;
-} /* sql_cursorDelLine */
-
-void sql_cursorInsLine(int cid, int rownum)
-{
-struct OCURS *o = findCursor(cid);
-short i;
-
-/* must insert a row within or immediately following the current data */
-if(rownum > o->alloc)
-errorPrint("2cursorInsLine(%d)", rownum);
-/* newly inserted row becomes the current row */
-o->rownum = rownum+1;
-
-if(!o->alloc || o->fl[o->alloc-1]) { /* need to make room */
-o->alloc += 128;
-if(!o->fl)
-o->fl = (char **) allocMem(o->alloc*sizeof(char*));
-else
-o->fl = (char**) reallocMem((void*)o->fl, o->alloc*sizeof(char*));
-memset(o->fl+o->alloc-128, 0, 128*sizeof(char*));
-} /* allocating more space */
-
-/* move the rest of the lines down */
-for(i=o->alloc-1; i>rownum; --i)
-o->fl[i] = o->fl[i-1];
-o->fl[i] = cloneString(sql_mkunld('\177'));
-} /* sql_cursorInsLine */
-
-
-/*********************************************************************
-run the analog of /bin/comm on two open cursors,
-rather than two Unix files.
-This assumes a common unique key that we use to sync up the rows.
-The cursors should be sorted by this key.
-*********************************************************************/
-
-void cursor_comm(
-const char *stmt1, const char *stmt2, /* the two select statements */
-const char *orderby, /* which fetched column is the unique key */
-fnptr f, /* call this function for differences */
-char delim) /* sql_mkunld() delimiter, or call mkinsupd if delim = 0 */
-{
-short cid1, cid2; /* the cursor ID numbers */
-char *line1, *line2, *s; /* the two fetched rows */
-void *blob1, *blob2; /* one blob per table */
-int blob1size, blob2size;
-bool eof1, eof2, get1, get2;
-int sortval1, sortval2;
-char sortstring1[80], sortstring2[80];
-int sortcol;
-char sorttype;
-int passkey1, passkey2;
-static const char sortnull[] = "cursor_comm, sortval%d is null";
-static const char sortlong[] = "cursor_comm cannot key on strings longer than %d";
-static const char noblob[] = "sorry, cursor_comm cannot handle blobs yet";
-
-cid1 = sql_prepOpen(stmt1);
-cid2 = sql_prepOpen(stmt2);
-
-sortcol = findcol_byname(orderby);
-sorttype = rv_type[sortcol];
-if(charInList("NDIS", sorttype) < 0)
-errorPrint("2cursor_com(), column %s has bad type %c", orderby, sorttype);
-if(sorttype == 'S')
-passkey1 = (int)sortstring1, passkey2 = (int)sortstring2;
-
-eof1 = eof2 = false;
-get1 = get2 = true;
-rv_blobFile = 0; /* in case the cursor has a blob */
-line1 = line2 = 0;
-blob1 = blob2 = 0;
-
-while(true) {
-if(get1) { /* fetch first row */
-eof1 = !sql_fetchNext(cid1, 0);
-nzFree(line1);
-line1 = 0;
-nzFree(blob1);
-blob1 = 0;
-if(!eof1) {
-if(sorttype == 'S') {
-s = rv_data[sortcol].ptr;
-if(isnullstring(s)) errorPrint(sortnull, 1);
-if(strlen(s) >= sizeof(sortstring1))
-errorPrint(sortlong, sizeof(sortstring1));
-strcpy(sortstring1, s);
-} else {
-passkey1 = sortval1 = rv_data[sortcol].l;
-if(isnull(sortval1))
-errorPrint(sortnull, 1);
-}
-line1 = cloneString(delim ? sql_mkunld(delim) : sql_mkinsupd());
-if(rv_blobLoc) {
-blob1 = rv_blobLoc;
-blob1size = rv_blobSize;
-errorPrint(noblob);
-}
-} /* not eof */
-} /* looking for first line */
-
-if(get2) { /* fetch second row */
-eof2 = !sql_fetchNext(cid2, 0);
-nzFree(line2);
-line2 = 0;
-nzFree(blob2);
-blob2 = 0;
-if(!eof2) {
-if(sorttype == 'S') {
-s = rv_data[sortcol].ptr;
-if(isnullstring(s)) errorPrint(sortnull, 2);
-if(strlen(s) >= sizeof(sortstring2))
-errorPrint(sortlong, sizeof(sortstring2));
-strcpy(sortstring2, rv_data[sortcol].ptr);
-} else {
-passkey2 = sortval2 = rv_data[sortcol].l;
-if(isnull(sortval2))
-errorPrint(sortnull, 2);
-}
-line2 = cloneString(delim ? sql_mkunld(delim) : sql_mkinsupd());
-if(rv_blobLoc) {
-blob2 = rv_blobLoc;
-blob2size = rv_blobSize;
-errorPrint(noblob);
-}
-} /* not eof */
-} /* looking for second line */
-
-if(eof1 & eof2) break; /* done */
-get1 = get2 = false;
-
-/* in cid2, but not in cid1 */
-if(eof1 || !eof2 &&
-(sorttype == 'S' && strcmp(sortstring1, sortstring2) > 0 ||
-sorttype != 'S' && sortval1 > sortval2)) {
-(*f)('>', line1, line2, passkey2);
-get2 = true;
-continue;
-}
-
-/* in cid1, but not in cid2 */
-if(eof2 || !eof1 &&
-(sorttype == 'S' && strcmp(sortstring1, sortstring2) < 0 ||
-sorttype != 'S' && sortval1 < sortval2)) {
-(*f)('<', line1, line2, passkey1);
-get1 = true;
-continue;
-} /* insert case */
-
-get1 = get2 = true;
-/* perhaps the lines are equal */
-if(stringEqual(line1, line2)) continue;
-
-/* lines are different between the two cursors */
-(*f)('*', line1, line2, passkey2);
-} /* loop over parallel cursors */
-
-nzFree(line1);
-nzFree(line2);
-nzFree(blob1);
-nzFree(blob2);
-sql_closeFree(cid1);
-sql_closeFree(cid2);
-} /* cursor_comm */
-
 /*********************************************************************
 Get the primary key for a table.
 In informix, you can use system tables to get this information.
-There's a way to do it in odbc, but I don't remember.
+I haven't yet expanded this to a 3 part key.
 *********************************************************************/
 
 void
-getPrimaryKey(char *tname, int *part1, int *part2)
+getPrimaryKey(char *tname, int *part1, int *part2, int *part3)
 {
-int p1, p2, rc;
-char *s = strchr(tname, ':');
-*part1 = *part2 = 0;
-if(!s) {
-rc = sql_select("select part1, part2 \
+    int p1, p2, rc;
+    char *s = strchr(tname, ':');
+    *part1 = *part2 = *part3 = 0;
+    if(!s) {
+	rc = sql_select("select part1, part2 \
 from sysconstraints c, systables t, sysindexes i \
 where tabname = %S and t.tabid = c.tabid \
-and constrtype = 'P' and c.idxname = i.idxname",
-tname, &p1, &p2);
-} else {
-*s = 0;
-rc = sql_select("select part1, part2 \
+and constrtype = 'P' and c.idxname = i.idxname", tname, &p1, &p2);
+    } else {
+	*s = 0;
+	rc = sql_select("select part1, part2 \
 from %s:sysconstraints c, %s:systables t, %s:sysindexes i \
 where tabname = %S and t.tabid = c.tabid \
-and constrtype = 'P' and c.idxname = i.idxname",
-tname, tname, tname, s+1, &p1, &p2);
-*s = ':';
-}
-if(rc) *part1 = p1, *part2 = p2;
-} /* getPrimaryKey */
+and constrtype = 'P' and c.idxname = i.idxname", tname, tname, tname, s + 1, &p1, &p2);
+	*s = ':';
+    }
+    if(rc)
+	*part1 = p1, *part2 = p2;
+}				/* getPrimaryKey */
 
+bool
+showTables(void)
+{
+puts("Not implemented in Informix, but certainly doable through systables");
+}				/* showTables */
 
-#line 1989 "dbinfx.ec"
+/* This can also be done by catalogs; it's on my list of things to do. */
+bool
+fetchForeign(char *tname)
+{
+i_puts(MSG_NYI);
+} /* fetchForeign */
+
+#line 1598 "dbinfx.ec"
