@@ -7,8 +7,77 @@
 #include "eb.h"
 #include "dbapi.h"
 
-const char *sql_debuglog = "ebsql.log";	/* log of debug prints */
+const char *sql_debuglog = "/tmp/ebsql.log";	/* log of debug prints */
 const char *sql_database;	/* name of current database */
+int rv_numRets;
+char rv_type[NUMRETS + 1];
+bool rv_nullable[NUMRETS];
+/* names of returned data, usually SQL column names */
+char rv_name[NUMRETS + 1][COLNAMELEN];
+LF rv_data[NUMRETS];		/* the returned values */
+long rv_lastNrows, rv_lastSerial, rv_lastRowid;
+void *rv_blobLoc;		/* location of blob in memory */
+int rv_blobSize;
+const char *rv_blobFile;
+bool rv_blobAppend;
+
+
+/* text descriptions corresponding to our generic SQL error codes */
+/* This has yet to be internationalized. */
+const char *sqlErrorList[] = { 0,
+    "miscelaneous SQL error",
+    "syntax error in SQL statement",
+    "filename cannot be used by SQL",
+    "cannot convert/compare the columns/constants in the SQL statement",
+    "bad string subscripting",
+    "bad use of the rowid construct",
+    "bad use of a blob column",
+    "bad use of aggregate operators or columns",
+    "bad use of a view",
+    "bad use of a serial column",
+    "bad use of a temp table",
+    "operation cannot cross databases",
+    "database is fucked up",
+    "query interrupted by user",
+    "could not connect to the database",
+    "database has not yet been selected",
+    "table not found",
+    "duplicate table",
+    "ambiguous table",
+    "column not found",
+    "duplicate column",
+    "ambiguous column",
+    "index not found",
+    "duplicate index",
+    "constraint not found",
+    "duplicate constraint",
+    "stored procedure not found",
+    "duplicate stored procedure",
+    "synonym not found",
+    "duplicate synonym",
+    "table has no primary or unique key",
+    "duplicate primary or unique key",
+    "cursor not specified, or cursor is not available",
+    "duplicate cursor",
+    "the database lacks the resources needed to complete this query",
+    "check constrain violated",
+    "referential integrity violated",
+    "cannot manage or complete the transaction",
+    "long transaction, too much log data generated",
+    "this operation must be run inside a transaction",
+    "cannot open, read, write, close, or otherwise manage a blob",
+    "row, table, page, or database is already locked, or cannot be locked",
+    "inserting null into a not null column",
+    "no permission to modify the database in this way",
+    "no current row established",
+    "many rows were found where one was expected",
+    "cannot union these select statements together",
+    "cannot access or write the audit trail",
+    "could not run SQL or gather data from a remote host",
+    "where clause is semantically unmanageable",
+    "deadlock detected",
+    0
+};
 
 char *
 lineFormat(const char *line, ...)
@@ -31,7 +100,7 @@ char *
 lineFormatStack(const char *line,	/* the sprintf-like formatting string */
    LF * argv,			/* pointer to array of values */
    va_list * parmv_p)
-{				/* pointers to parameters on the stack */
+{
     va_list parmv = 0;
     short i, len, maxlen, len_given, flags;
     long n;
@@ -597,24 +666,21 @@ stringMoney(const char *s)
 }				/* stringMoney */
 
 /* Make sure edbrowse is connected to the database */
-static bool
+bool
 ebConnect(void)
 {
-    const short exclist[] = { EXCSQLMISC, EXCNOCONNECT, 0 };
     if(sql_database)
 	return true;
     if(!dbarea) {
 	setError(MSG_DBUnspecified);
 	return false;
     }
-    sql_exclist(exclist);
     sql_connect(dbarea, dblogin, dbpw);
-    if(rv_lastStatus) {
+    if(!sql_database) {
 	setError(MSG_DBConnect, rv_vendorStatus);
 	return false;
     }
-    if(!sql_database)
-	errorPrint("@sql connected, but database not set");
+
     return true;
 }				/* ebConnect */
 
@@ -633,11 +699,50 @@ static int wcllen;
 static char wherecol[COLNAMELEN + 2];
 static struct DBTABLE *td;
 
+/* put quotes around a column value */
 static void
-unexpected(void)
+pushQuoted(char **s, int *slen, const char *value, int colno)
 {
-    setError(MSG_DBUnexpected, rv_vendorStatus);
-}				/* unexpected */
+    char quotemark = 0;
+    char coltype = td->types[colno];
+    if(coltype != 'F' && coltype != 'N') {
+/* Microsoft insists on single quote. */
+	quotemark = '\'';
+	if(strchr(value, quotemark))
+	    quotemark = '"';
+    }
+    if(quotemark)
+	stringAndChar(s, slen, quotemark);
+    stringAndString(s, slen, value);
+    if(quotemark)
+	stringAndChar(s, slen, quotemark);
+}				/* pushQuoted */
+
+static char *lineFields[MAXTCOLS];
+
+static char *
+keysQuoted(void)
+{
+    char *u;
+    int ulen;
+    int key1 = td->key1, key2 = td->key2;
+    if(!key1)
+	return 0;
+    u = initString(&ulen);
+    --key1;
+    stringAndString(&u, &ulen, "where ");
+    stringAndString(&u, &ulen, td->cols[key1]);
+    stringAndString(&u, &ulen, " = ");
+    pushQuoted(&u, &ulen, lineFields[key1], key1);
+    if(!key2)
+	return u;
+    --key2;
+    stringAndString(&u, &ulen, " and ");
+    stringAndString(&u, &ulen, td->cols[key2]);
+    stringAndString(&u, &ulen, " = ");
+    pushQuoted(&u, &ulen, lineFields[key2], key2);
+    return u;
+}				/* keysQuoted */
 
 static void
 buildSelectClause(void)
@@ -657,7 +762,7 @@ buildSelectClause(void)
 static bool
 buildWhereClause(void)
 {
-    int i, l, n;
+    int i, l, n, colno;
     const char *w = myWhere;
     const char *e;
 
@@ -672,7 +777,8 @@ buildWhereClause(void)
 	    setError(MSG_DBNoKey);
 	    return false;
 	}
-	e = td->cols[td->key1 - 1];
+	colno = td->key1;
+	e = td->cols[colno - 1];
 	l = strlen(e);
 	if(l > COLNAMELEN) {
 	    setError(MSG_DBColumnLong, e, COLNAMELEN);
@@ -681,37 +787,37 @@ buildWhereClause(void)
 	strcpy(wherecol, e);
 	e = w - 1;
     } else if(isdigit(*w)) {
-	n = strtol(w, (char **)&w, 10);
+	colno = strtol(w, (char **)&w, 10);
 	if(w != e) {
 	    setError(MSG_DBSyntax);
 	    return false;
 	}
-	if(n == 0 || n > td->ncols) {
-	    setError(MSG_DBColRange, n);
+	if(colno == 0 || colno > td->ncols) {
+	    setError(MSG_DBColRange, colno);
 	    return false;
 	}
 	goto setcol_n;
     } else {
-	n = 0;
+	colno = 0;
 	if(e - w <= COLNAMELEN) {
 	    strncpy(wherecol, w, e - w);
 	    wherecol[e - w] = 0;
 	    for(i = 0; i < td->ncols; ++i) {
 		if(!strstr(td->cols[i], wherecol))
 		    continue;
-		if(n) {
+		if(colno) {
 		    setError(MSG_DBManyColumns, wherecol);
 		    return false;
 		}
-		n = i + 1;
+		colno = i + 1;
 	    }
 	}
-	if(!n) {
+	if(!colno) {
 	    setError(MSG_DBNoColumn, wherecol);
 	    return false;
 	}
       setcol_n:
-	w = td->cols[n - 1];
+	w = td->cols[colno - 1];
 	l = strlen(w);
 	if(l > COLNAMELEN) {
 	    setError(MSG_DBColumnLong, w, COLNAMELEN);
@@ -735,7 +841,8 @@ buildWhereClause(void)
     } else if(w[strlen(w) - 1] == '*') {
 	stringAndString(&wcl, &wcllen, lineFormat(" matches %S", w));
     } else {
-	stringAndString(&wcl, &wcllen, lineFormat(" = %S", w));
+	stringAndString(&wcl, &wcllen, " = ");
+	pushQuoted(&wcl, &wcllen, w, colno - 1);
     }
 
     return true;
@@ -744,8 +851,8 @@ buildWhereClause(void)
 static bool
 setTable(void)
 {
-    static const short exclist[] = { EXCNOTABLE, EXCNOCOLUMN, EXCSQLMISC, 0 };
-    int cid, nc, i, part1, part2;
+    static const short exclist[] = { EXCNOTABLE, EXCNOCOLUMN, 0 };
+    int cid, nc, i, part1, part2, part3;
     const char *s = cw->fileName;
     const char *t = strchr(s, ']');
     if(t - s >= sizeof (myTab))
@@ -772,8 +879,6 @@ setTable(void)
 		    setError(MSG_DBNoTable, td->name);
 		else if(rv_lastStatus == EXCNOCOLUMN)
 		    setError(MSG_DBBadColumn);
-		else
-		    unexpected();
 		return false;
 	    }
 	    td->types = cloneString(rv_type);
@@ -787,8 +892,6 @@ setTable(void)
 	if(rv_lastStatus) {
 	    if(rv_lastStatus == EXCNOTABLE)
 		setError(MSG_DBNoTable, myTab);
-	    else
-		unexpected();
 	    return false;
 	}
 	td = newTableDescriptor(myTab);
@@ -809,21 +912,16 @@ setTable(void)
 	    td->cols[i] = cloneString(rv_name[i]);
 	sql_free(cid);
 
-	getPrimaryKey(myTab, &part1, &part2);
+	getPrimaryKey(myTab, &part1, &part2, &part3);
 	if(part1 > nc)
 	    part1 = 0;
 	if(part2 > nc)
 	    part2 = 0;
+	if(part3 > nc)
+	    part3 = 0;
 	td->key1 = part1;
 	td->key2 = part2;
-    }
-
-    s = strpbrk(td->types, "BT");
-    if(s)
-	s = strpbrk(s + 1, "BT");
-    if(s) {
-	setError(MSG_DBManyBlobs);
-	return false;
+	td->key3 = part3;
     }
 
     cw->table = td;
@@ -839,15 +937,18 @@ showColumns(void)
 
     if(!setTable())
 	return;
-    printf("table %s", td->name);
+    i_printf(MSG_Table);
+    printf(" %s", td->name);
     if(!stringEqual(td->name, td->shortname))
 	printf(" [%s]", td->shortname);
     i = sql_selectOne("select count(*) from %s", td->name);
-    printf(", %d rows\n", i);
+    printf(", %d ", i);
+    i_printf(i == 1 ? MSG_Row : MSG_Rows);
+    nl();
 
     for(i = 0; i < td->ncols; ++i) {
 	printf("%d ", i + 1);
-	if(td->key1 == i + 1 || td->key2 == i + 1)
+	if(td->key1 == i + 1 || td->key2 == i + 1 || td->key3 == i + 1)
 	    printf("*");
 	printf("%s ", td->cols[i]);
 	c = td->types[i];
@@ -887,12 +988,79 @@ showColumns(void)
     }
 }				/* showColumns */
 
+void
+showForeign(void)
+{
+    if(!setTable())
+	return;
+    i_printf(MSG_Fkeys, td->name);
+    fetchForeign(td->name);
+}				/* showForeign */
+
+/* Select rows of data and put them into the text buffer */
+static bool
+rowsIntoBuffer(int cid, const char *types, char **bufptr, int *lcnt)
+{
+    char *rbuf, *unld, *u, *v, *s, *end;
+    int rbuflen;
+    bool rc = false;
+
+    *bufptr = EMPTYSTRING;
+    *lcnt = 0;
+    rbuf = initString(&rbuflen);
+
+    while(sql_fetchNext(cid, 0)) {
+	unld = sql_mkunld('\177');
+	if(strchr(unld, '|')) {
+	    setError(MSG_DBPipes);
+	    goto abort;
+	}
+	if(strchr(unld, '\n')) {
+	    setError(MSG_DBNewline);
+	    goto abort;
+	}
+	for(s = unld; *s; ++s)
+	    if(*s == '\177')
+		*s = '|';
+	s[-1] = '\n';		/* overwrite the last pipe */
+
+/* look for blob column */
+	if(rv_blobLoc && (s = strpbrk(types, "BT"))) {
+	    int bfi = s - types;	/* blob field index */
+	    int cx = 0;		/* context, where to put the blob */
+	    int j;
+
+	    u = unld;
+	    for(j = 0; j < bfi; ++j)
+		u = strchr(u, '|') + 1;
+	    v = strpbrk(u, "|\n");
+	    end = v + strlen(v);
+	    cx = sideBuffer(0, rv_blobLoc, rv_blobSize, 0, false);
+	    nzFree(rv_blobLoc);
+	    sprintf(myTab, "<%d>", cx);
+	    if(!cx)
+		myTab[0] = 0;
+	    j = strlen(myTab);
+/* unld is pretty long; I'm just going to assume there is enough room for this */
+	    memmove(u + j, v, end + 1 - v);
+	    memcpy(u, myTab, j);
+	}
+
+	stringAndString(&rbuf, &rbuflen, unld);
+	++*lcnt;
+    }
+    rc = true;
+
+  abort:
+    sql_closeFree(cid);
+    *bufptr = rbuf;
+    return rc;
+}				/* rowsIntoBuffer */
+
 bool
 sqlReadRows(const char *filename, char **bufptr)
 {
-    int cid;
-    char *rbuf, *unld, *s;
-    int rlen;
+    int cid, lcnt;
 
     *bufptr = EMPTYSTRING;
     if(!ebConnect())
@@ -900,72 +1068,23 @@ sqlReadRows(const char *filename, char **bufptr)
     if(!setTable())
 	return false;
 
-    rbuf = initString(&rlen);
     myWhere = strchr(filename, ']') + 1;
-    if(*myWhere) {
-	if(!buildWhereClause())
-	    return false;
-	buildSelectClause();
-	rv_blobFile = 0;
-	cid = sql_prepOpen("%s %0s", scl, wcl);
-	nzFree(scl);
-	nzFree(wcl);
-	while(sql_fetchNext(cid, 0)) {
-	    unld = sql_mkunld('\177');
-	    if(strchr(unld, '|')) {
-		setError(MSG_DBPipes);
-		goto abort;
-	    }
-	    if(strchr(unld, '\n')) {
-		setError(MSG_DBNewline);
-		goto abort;
-	    }
-	    for(s = unld; *s; ++s)
-		if(*s == '\177')
-		    *s = '|';
-	    s[-1] = '\n';	/* overwrite the last pipe */
+    if(!*myWhere)
+	return true;
 
-/* look for blob column */
-	    if(s = strpbrk(td->types, "BT")) {
-		int bfi = s - td->types;	/* blob field index */
-		int cx = 0;	/* context, where to put the blob */
-		int j;
-		char *u, *v, *end;
-		u = unld;
-		for(j = 0; j < bfi; ++j)
-		    u = strchr(u, '|') + 1;
-		v = strpbrk(u, "|\n");
-		end = v + strlen(v);
-		if(rv_blobSize) {
-		    cx = sideBuffer(0, rv_blobLoc, rv_blobSize, 0, false);
-		    nzFree(rv_blobLoc);
-		}
-		sprintf(myTab, "<%d>", cx);
-		if(!cx)
-		    myTab[0] = 0;
-		j = strlen(myTab);
-/* unld is pretty long; I'm just going to assume there is enough room for this */
-		memmove(u + j, v, end - v);
-		u[j + (end - v)] = 0;
-		memcpy(u, myTab, j);
-	    }
+    if(!buildWhereClause())
+	return false;
+    buildSelectClause();
+    rv_blobFile = 0;
+    cid = sql_prepOpen("%s %0s", scl, wcl);
+    nzFree(scl);
+    nzFree(wcl);
+    if(cid < 0)
+	return false;
 
-	    stringAndString(&rbuf, &rlen, unld);
-	}
-	sql_closeFree(cid);
-    }
-
-    *bufptr = rbuf;
-    return true;
-
-  abort:
-    nzFree(rbuf);
-    sql_closeFree(cid);
-    return false;
+    return rowsIntoBuffer(cid, td->types, bufptr, &lcnt);
 }				/* sqlReadRows */
 
-static char *lineFields[MAXTCOLS];
-
 /* Split a line at pipe boundaries, and make sure the field count is correct */
 static bool
 intoFields(char *line)
@@ -1013,11 +1132,16 @@ keyCountCheck(void)
 	setError(MSG_DBNoKeyCol);
 	return false;
     }
-    return (td->key2 ? 2 : 1);
+    if(!td->key2)
+	return 1;
+    if(!td->key3)
+	return 2;
+    setError(MSG_DBManyKeyCol);
+    return 0;
 }				/* keyCountCheck */
 
 /* Typical error conditions for insert update delete */
-static const short insupdExceptions[] = { EXCSQLMISC,
+static const short insupdExceptions[] = {
     EXCVIEWUSE, EXCREFINT, EXCITEMLOCK, EXCPERMISSION,
     EXCDEADLOCK, EXCCHECK, EXCTIMEOUT, EXCNOTNULLCOLUMN, 0
 };
@@ -1069,14 +1193,12 @@ insupdError(int action, int rcnt)
 bool
 sqlDelRows(int start, int end)
 {
-    int nkeys, ndel, key1, key2, ln;
+    int nkeys, ndel, ln;
 
     if(!setTable())
 	return false;
 
     nkeys = keyCountCheck();
-    key1 = td->key1 - 1;
-    key2 = td->key2 - 1;
     if(!nkeys)
 	return false;
 
@@ -1092,16 +1214,13 @@ sqlDelRows(int start, int end)
  * I have to write the one-line-at-a-time code anyways,
  * I'll just use that for now. */
     while(ndel--) {
+	char *wherekeys;
 	char *line = (char *)fetchLine(ln, 0);
 	intoFields(line);
+	wherekeys = keysQuoted();
 	sql_exclist(insupdExceptions);
-	if(nkeys == 1)
-	    sql_exec("delete from %s where %s = %S",
-	       td->name, td->cols[key1], lineFields[key1]);
-	else
-	    sql_exec("delete from %s where %s = %S and %s = %S",
-	       td->name, td->cols[key1], lineFields[key1],
-	       td->cols[key2], lineFields[key2]);
+	sql_exec("delete from %s %s", td->name, wherekeys);
+	nzFree(wherekeys);
 	nzFree(line);
 	if(!insupdError(0, 1))
 	    return false;
@@ -1115,10 +1234,11 @@ bool
 sqlUpdateRow(pst source, int slen, pst dest, int dlen)
 {
     char *d2;			/* clone of dest */
+    char *wherekeys;
     char *s, *t;
     int j, l1, l2, nkeys, key1, key2;
-    char *u1, *u2;		/* pieces of the update statement */
-    int u1len, u2len;
+    char *u1;			/* column=value of the update statement */
+    int u1len;
 
 /* compare all the way out to newline, so we know both strings end at the same time */
     if(slen == dlen && !memcmp(source, dest, slen + 1))
@@ -1128,10 +1248,10 @@ sqlUpdateRow(pst source, int slen, pst dest, int dlen)
 	return false;
 
     nkeys = keyCountCheck();
-    key1 = td->key1 - 1;
-    key2 = td->key2 - 1;
     if(!nkeys)
 	return false;
+    key1 = td->key1 - 1;
+    key2 = td->key2 - 1;
 
     d2 = (char *)clonePstring(dest);
     if(!intoFields(d2)) {
@@ -1141,7 +1261,6 @@ sqlUpdateRow(pst source, int slen, pst dest, int dlen)
 
     j = 0;
     u1 = initString(&u1len);
-    u2 = initString(&u2len);
     s = (char *)source;
 
     while(1) {
@@ -1162,38 +1281,32 @@ sqlUpdateRow(pst source, int slen, pst dest, int dlen)
 		goto abort;
 	    }
 	    if(*u1)
-		stringAndChar(&u1, &u1len, ',');
+		stringAndString(&u1, &u1len, ", ");
 	    stringAndString(&u1, &u1len, td->cols[j]);
-	    if(*u2)
-		stringAndChar(&u2, &u2len, ',');
-	    stringAndString(&u2, &u2len, lineFormat("%S", lineFields[j]));
+	    stringAndString(&u1, &u1len, " = ");
+	    pushQuoted(&u1, &u1len, lineFields[j], j);
 	}
+
 	if(*t == '\n')
 	    break;
 	s = t + 1;
 	++j;
     }
 
+    wherekeys = keysQuoted();
     sql_exclist(insupdExceptions);
-    if(nkeys == 1)
-	sql_exec("update %s set(%s) = (%s) where %s = %S",
-	   td->name, u1, u2, td->cols[key1], lineFields[key1]);
-    else
-	sql_exec("update %s set(%s) = (%s) where %s = %S and %s = %S",
-	   td->name, u1, u2,
-	   td->cols[key1], lineFields[key1], td->cols[key2], lineFields[key2]);
+    sql_exec("update %s set %s %s", td->name, u1, wherekeys);
+    nzFree(wherekeys);
     if(!insupdError(2, 1))
 	goto abort;
 
     nzFree(d2);
     nzFree(u1);
-    nzFree(u2);
     return true;
 
   abort:
     nzFree(d2);
     nzFree(u1);
-    nzFree(u2);
     return false;
 }				/* sqlUpdateRow */
 
@@ -1201,18 +1314,23 @@ bool
 sqlAddRows(int ln)
 {
     char *u1, *u2;		/* pieces of the insert statement */
+    char *u3;			/* line with pipes */
     char *unld, *s;
-    int u1len, u2len;
-    int j, l, rowid;
+    int u1len, u2len, u3len;
+    int j, l, nkeys;
     double dv;
     char inp[256];
+    bool rc;
 
     if(!setTable())
 	return false;
+    nkeys = keyCountCheck();
 
     while(1) {
 	u1 = initString(&u1len);
 	u2 = initString(&u2len);
+	u3 = initString(&u3len);
+
 	for(j = 0; j < td->ncols; ++j) {
 	  reenter:
 	    if(strchr("BT", td->types[j]))
@@ -1229,13 +1347,17 @@ sqlAddRows(int ln)
 	    if(stringEqual(inp, ".")) {
 		nzFree(u1);
 		nzFree(u2);
+		nzFree(u3);
 		return true;
 	    }
 
-/* For now, a null field is always excepted. */
-/* Someday we may want to check this against the not-null constraint. */
-	    if(inp[0] == 0)
+	    if(inp[0] == 0) {
+/* I thought it was a good idea to prevent nulls from going into not-null
+ * columns, but then I remembered  not null default value,
+ * where the database converts null into something real.
+ * I want to allow this. */
 		goto goodfield;
+	    }
 
 /* verify the integrity of the entered field */
 	    if(strchr(inp, '|')) {
@@ -1280,24 +1402,44 @@ sqlAddRows(int ln)
 	    }
 
 	  goodfield:
+
+/* turn 0 into next serial number */
+	    if(j == td->key1 - 1 && td->types[j] == 'N' &&
+	       stringEqual(inp, "0")) {
+		int nextkey = sql_selectOne("select max(%s) from %s",
+		   td->cols[j], td->name);
+		if(isnull(nextkey)) {
+		    i_puts(MSG_DBNextSerial);
+		    goto reenter;
+		}
+		sprintf(inp, "%d", nextkey + 1);
+	    }
+
 	    if(*u1)
 		stringAndChar(&u1, &u1len, ',');
 	    stringAndString(&u1, &u1len, td->cols[j]);
+
 	    if(*u2)
 		stringAndChar(&u2, &u2len, ',');
-	    stringAndString(&u2, &u2len, lineFormat("%S", inp));
+	    pushQuoted(&u2, &u2len, inp, j);
+
+	    stringAndString(&u3, &u3len, inp);
+	    stringAndChar(&u3, &u3len, '|');
 	}
+
 	sql_exclist(insupdExceptions);
 	sql_exec("insert into %s (%s) values (%s)", td->name, u1, u2);
 	nzFree(u1);
 	nzFree(u2);
 	if(!insupdError(1, 1)) {
+	    nzFree(u3);
 	    printf("Error: ");
 	    showError();
 	    continue;
 	}
-/* Fetch the row just entered;
-its serial number may have changed from 0 to something real */
+#if 0
+/* Fetch the row just entered. */
+/* Don't know how to do this without rowid. */
 	rowid = rv_lastRowid;
 	buildSelectClause();
 	sql_select("%s where rowid = %d", scl, rowid, 0);
@@ -1305,7 +1447,15 @@ its serial number may have changed from 0 to something real */
 	unld = sql_mkunld('|');
 	l = strlen(unld);
 	unld[l - 1] = '\n';	/* overwrite the last pipe */
-	if(!addTextToBuffer((pst) unld, l, ln))
+#else
+	unld = u3;
+	l = strlen(unld);
+	unld[l - 1] = '\n';	/* overwrite the last pipe */
+#endif
+
+	rc = addTextToBuffer((pst) unld, l, ln, false);
+	nzFree(u3);
+	if(!rc)
 	    return false;
 	++ln;
     }
@@ -1315,6 +1465,148 @@ its serial number may have changed from 0 to something real */
 }				/* sqlAddRows */
 
 
+/*********************************************************************
+run the analog of /bin/comm on two open cursors,
+rather than two Unix files.
+This assumes a common unique key that we use to sync up the rows.
+The cursors should be sorted by this key.
+*********************************************************************/
+
+static void
+cursor_comm(const char *stmt1, const char *stmt2,	/* the two select statements */
+   const char *orderby,		/* which fetched column is the unique key */
+   fnptr f,			/* call this function for differences */
+   char delim)
+{				/* sql_mkunld() delimiter, or call mkinsupd if delim = 0 */
+    short cid1, cid2;		/* the cursor ID numbers */
+    char *line1, *line2, *s;	/* the two fetched rows */
+    void *blob1, *blob2;	/* one blob per table */
+    int blob1size, blob2size;
+    bool eof1, eof2, get1, get2;
+    int sortval1, sortval2;
+    char sortstring1[80], sortstring2[80];
+    int sortcol;
+    char sorttype;
+    int passkey1, passkey2;
+    static const char sortnull[] = "cursor_comm, sortval%d is null";
+    static const char sortlong[] =
+       "cursor_comm cannot key on strings longer than %d";
+    static const char noblob[] = "sorry, cursor_comm cannot handle blobs yet";
+
+    cid1 = sql_prepOpen(stmt1);
+    cid2 = sql_prepOpen(stmt2);
+
+    sortcol = findColByName(orderby);
+    sorttype = rv_type[sortcol];
+    if(charInList("NDIS", sorttype) < 0)
+	errorPrint("2cursor_com(), column %s has bad type %c", orderby,
+	   sorttype);
+    if(sorttype == 'S')
+	passkey1 = (int)sortstring1, passkey2 = (int)sortstring2;
+
+    eof1 = eof2 = false;
+    get1 = get2 = true;
+    rv_blobFile = 0;		/* in case the cursor has a blob */
+    line1 = line2 = 0;
+    blob1 = blob2 = 0;
+
+    while(true) {
+	if(get1) {		/* fetch first row */
+	    eof1 = !sql_fetchNext(cid1, 0);
+	    nzFree(line1);
+	    line1 = 0;
+	    nzFree(blob1);
+	    blob1 = 0;
+	    if(!eof1) {
+		if(sorttype == 'S') {
+		    s = rv_data[sortcol].ptr;
+		    if(isnullstring(s))
+			errorPrint(sortnull, 1);
+		    if(strlen(s) >= sizeof (sortstring1))
+			errorPrint(sortlong, sizeof (sortstring1));
+		    strcpy(sortstring1, s);
+		} else {
+		    passkey1 = sortval1 = rv_data[sortcol].l;
+		    if(isnull(sortval1))
+			errorPrint(sortnull, 1);
+		}
+		line1 = cloneString(delim ? sql_mkunld(delim) : sql_mkinsupd());
+		if(rv_blobLoc) {
+		    blob1 = rv_blobLoc;
+		    blob1size = rv_blobSize;
+		    errorPrint(noblob);
+		}
+	    }			/* not eof */
+	}
+	/* looking for first line */
+	if(get2) {		/* fetch second row */
+	    eof2 = !sql_fetchNext(cid2, 0);
+	    nzFree(line2);
+	    line2 = 0;
+	    nzFree(blob2);
+	    blob2 = 0;
+	    if(!eof2) {
+		if(sorttype == 'S') {
+		    s = rv_data[sortcol].ptr;
+		    if(isnullstring(s))
+			errorPrint(sortnull, 2);
+		    if(strlen(s) >= sizeof (sortstring2))
+			errorPrint(sortlong, sizeof (sortstring2));
+		    strcpy(sortstring2, rv_data[sortcol].ptr);
+		} else {
+		    passkey2 = sortval2 = rv_data[sortcol].l;
+		    if(isnull(sortval2))
+			errorPrint(sortnull, 2);
+		}
+		line2 = cloneString(delim ? sql_mkunld(delim) : sql_mkinsupd());
+		if(rv_blobLoc) {
+		    blob2 = rv_blobLoc;
+		    blob2size = rv_blobSize;
+		    errorPrint(noblob);
+		}
+	    }			/* not eof */
+	}
+	/* looking for second line */
+	if(eof1 & eof2)
+	    break;		/* done */
+	get1 = get2 = false;
+
+/* in cid2, but not in cid1 */
+	if(eof1 || !eof2 &&
+	   (sorttype == 'S' && strcmp(sortstring1, sortstring2) > 0 ||
+	   sorttype != 'S' && sortval1 > sortval2)) {
+	    (*f) ('>', line1, line2, passkey2);
+	    get2 = true;
+	    continue;
+	}
+
+/* in cid1, but not in cid2 */
+	if(eof2 || !eof1 &&
+	   (sorttype == 'S' && strcmp(sortstring1, sortstring2) < 0 ||
+	   sorttype != 'S' && sortval1 < sortval2)) {
+	    (*f) ('<', line1, line2, passkey1);
+	    get1 = true;
+	    continue;
+	}
+	/* insert case */
+	get1 = get2 = true;
+/* perhaps the lines are equal */
+	if(stringEqual(line1, line2))
+	    continue;
+
+/* lines are different between the two cursors */
+	(*f) ('*', line1, line2, passkey2);
+    }				/* loop over parallel cursors */
+
+    nzFree(line1);
+    nzFree(line2);
+    nzFree(blob1);
+    nzFree(blob2);
+    sql_closeFree(cid1);
+    sql_closeFree(cid2);
+}				/* cursor_comm */
+
+
 /*********************************************************************
 Sync up two tables, or corresponding sections of two tables.
 These are usually equischema tables in parallel databases or machines.
@@ -1330,6 +1622,21 @@ static const char *synctable;	/* table being sync-ed */
 static const char *synckeycol;	/* key column */
 static const char *sync_clause;	/* additional clause, to sync only part of the table */
 
+/* convert column name into column index */
+int
+findColByName(const char *name)
+{
+    int i;
+    for(i = 0; rv_name[i][0]; ++i)
+	if(stringEqual(name, rv_name[i]))
+	    break;
+    if(!rv_name[i][0])
+	errorPrint
+	   ("2Column %s not found in the columns or aliases of your select statement",
+	   name);
+    return i;
+}				/* findColByName */
+
 static int
 syncup_comm_fn(char action, char *line1, char *line2, int key)
 {
@@ -1388,3 +1695,110 @@ syncup_table(const char *table1, const char *table2,	/* the two tables */
 
     cursor_comm(stmt1, stmt2, keycol, (fnptr) syncup_comm_fn, 0);
 }				/* syncup_table */
+
+int
+goSelect(int *startLine, char **rbuf)
+{
+    int lineno = *startLine;
+    pst line;
+    char *cmd, *s;
+    int cmdlen;
+    int i, j, l, action, cid;
+    bool rc;
+    static const char *actionWords[] = {
+	"select", "insert", "update", "delete", "execute",
+	0
+    };
+    static const actionCodes[] = {
+	MSG_Selected, MSG_Inserted, MSG_Updated, MSG_Deleted, MSG_ProcExec
+    };
+
+    *rbuf = EMPTYSTRING;
+
+/* Make sure first line begins with ] */
+    line = fetchLine(lineno, -1);
+    if(!line || line[0] != ']')
+	return -1;
+
+    j = pstLength(line);
+    cmd = initString(&cmdlen);
+    stringAndBytes(&cmd, &cmdlen, line, j);
+    cmd[0] = ' ';
+
+    while(j == 1 || line[j - 2] != ';') {
+	if(++lineno > cw->dol || !(line = fetchLine(lineno, -1))) {
+	    setError(MSG_UnterminatedSelect);
+	    nzFree(cmd);
+	    return 0;
+	}
+	if(line[0] == ']') {
+	    --lineno;
+	    break;
+	}
+	j = pstLength(line);
+	stringAndBytes(&cmd, &cmdlen, line, j);
+    }
+
+/* Try to infer action from the first word of the command. */
+    action = -1;
+    for(s = cmd; isspaceByte(*s); ++s) ;
+    for(i = 0; actionWords[i]; ++i) {
+	l = strlen(actionWords[i]);
+	if(memEqualCI(s, actionWords[i], l) && isspace(s[l])) {
+	    action = actionCodes[i];
+	    break;
+	}
+    }
+
+    if(!ebConnect()) {
+	nzFree(cmd);
+	return 0;
+    }
+
+    rv_blobFile = 0;
+
+    if(action == MSG_Selected) {
+	cid = sql_prepOpen(cmd);
+	nzFree(cmd);
+	if(cid < 0)
+	    return 0;
+      grabrows:
+	*startLine = lineno;
+	rc = rowsIntoBuffer(cid, rv_type, rbuf, &j);
+      printrows:
+	printf("%d ", j);
+	i_printf(j == 1 ? MSG_Row : MSG_Rows);
+	printf(" ");
+	i_printf(action);
+	nl();
+	return rc;
+    }
+
+    if(action == MSG_ProcExec) {
+	cid = sql_prepOpen(cmd);
+	nzFree(cmd);
+	if(cid < 0)
+	    return 0;
+	if(rv_numRets) {
+	    action = MSG_Selected;
+	    goto grabrows;
+	}
+	sql_closeFree(cid);
+	*startLine = lineno;
+	i_puts(MSG_ProcExec);
+	return 1;
+    }
+
+/* Don't know what kind of sql command this is. */
+/* Run it anyways. */
+    rc = sql_execNF(cmd);
+    nzFree(cmd);
+    j = rv_lastNrows;
+    if(rc)
+	*startLine = lineno;
+    if(action >= MSG_Selected && action <= MSG_Deleted && (j || rc))
+	goto printrows;
+    if(rc)
+	i_puts(MSG_OK);
+    return rc;
+}				/* goSelect */
