@@ -30,6 +30,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <curl/curl.h>
 #ifdef DOSLIKE
 #include <io.h>
 #else
@@ -48,6 +49,16 @@
 #define O_SYNC 0
 #endif
 
+/* Some error constants that aren't defined in older versions of libcurl. 
+ * I'm not too worried about them, because the catch-all handles so many
+ * things..  Plus, everyone will eventually be using the latest libcurl. */
+#ifndef CURLE_PEER_FAILED_VERIFICATION
+#define CURLE_PEER_FAILED_VERIFICATION -1
+#endif
+#ifndef CURLE_REMOTE_ACCESS_DENIED
+#define CURLE_REMOTE_ACCESS_DENIED -2
+#endif
+
 /* WARNING:  the following typedef is pseudo-standard in C.
  * Some systems will define ushort in sys/types.h, others will not.
  * Unfortunately there is no #define symbol to key on;
@@ -104,8 +115,6 @@ typedef uchar *pst;		/* perl string */
 #define MAXTTYLINE 256
 /* Longest line that can take the substitute command */
 #define REPLACELINELEN 50000
-/* How many pages can we fetch, through frames etc. */
-#define MAXFETCH 100
 /* The longest string, in certain contexts. */
 #define MAXSTRLEN 1024
 /* How about user login and password? */
@@ -136,11 +145,6 @@ typedef uchar *pst;		/* perl string */
 #define COLOR_DIALOG_TEXT 0
 #define G_BFU_FONT_SIZE 0
 
-/* .trash is small or capital t, on different systems */
-#ifndef TRASHDIR
-#define TRASHDIR ".Trash"
-#endif
-
 /* the version of edbrowse */
 extern const char *version;
 extern const char eol[];	/* internet end of line */
@@ -174,17 +178,19 @@ struct DBTABLE {
     char *name, *shortname;
     char *cols[MAXTCOLS];
     char ncols;
-    char key1, key2;
+    unsigned char key1, key2, key3;
     char *types;
 };
 
 /* various globals */
+extern CURL *curl_handle;
 extern int debugLevel;		/* 0 to 9 */
 extern int webTimeout, mailTimeout;
 extern int browseLine;		/* line number, for error reporting */
 extern bool ismc;		/* Is the program running as a mail client? */
 extern bool cons_utf8;		/* does the console expect utf8? */
 extern bool iuConvert;		/* perform iso utf8 conversions automatically */
+extern char type8859;		/* 1 through 15 */
 extern bool zapMail;		/* just get rid of the mail */
 extern bool js_redirects;	/* window.location = new_url */
 extern uchar browseLocal;	/* browsing a local file */
@@ -217,16 +223,15 @@ extern struct MACCOUNT accounts[];	/* all the email accounts */
 extern int maxMime;		/* how many mime types specified */
 extern struct MIMETYPE mimetypes[];
 extern char *dbarea, *dblogin, *dbpw;	/* to log into the database */
+extern bool fetchBlobColumns;
 extern char *proxy_host;
 extern int proxy_port;
 extern bool caseInsensitive, searchStringsAll;
-extern bool textAreaDosNewlines;	/* when transmitting a textarea */
 extern bool undoable;		/* an undoable operation is taking place */
 extern bool allowRedirection;	/* from http code 301, or http refresh */
 extern bool sendReferrer;	/* in the http header */
 extern bool allowJS;		/* javascript on */
 extern bool helpMessagesOn;	/* no need to type h */
-extern char ftpMode;
 extern bool showHiddenFiles;	/* during directory scan */
 extern uchar dirWrite;		/* directory write mode, e.g. rename files */
 extern uchar endMarks;		/* do we print ^ $ at the start and end of lines? */
@@ -311,6 +316,7 @@ struct ebWindow {
 /* remember dot and dol for the raw text, when in browse mode */
     int r_dot, r_dol;
     char *fileName;		/* name of file or url */
+    char *firstURL;		/* before http redirection */
     char *referrer;
     char *baseDirName;		/* when scanning a directory */
     char *ft, *fd, *fk;		/* title, description, keywords */
