@@ -29,10 +29,6 @@ prepareForBrowse(char *h, int h_len)
 		--j;
 	    continue;
 	}
-	if(h[i] == (char)0xe2 && i < h_len - 1 && h[i + 1] == (char)0x80) {
-	    ++i;
-	    continue;
-	}
 	if(h[i] == InternalCodeChar)
 	    h[i] = InternalCodeCharAlternate;
 	h[j++] = h[i];
@@ -414,8 +410,9 @@ anchorSwap(char *buf)
     char tag[20];
 
     static const char from[] =
-       "\x1b\x95\x99\x9c\x9d\x91\x92\x93\x94\xa0\xad\x96\x97\x85\xa6\xc2";
-    static const char becomes[] = "_*'`'`'`' ----- ";
+       "\x1b\x95\x99\x9c\x9d\x91\x92\x93\x94\xa0\xad\x96\x97\x85";
+    static const char becomes[] = "_*'`'`'`' ----";
+/* I use to convert a6 and c2 to hyphen space, not sure why */
 
 /* Transliterate a few characters.  One of them is 0xa0 to space,
  * so we need to do this now, before the anchors swap with whitespace.
@@ -1060,7 +1057,7 @@ htmlReformat(const char *buf)
 	    }
 	    continue;
 	}
-	/* white space */
+
 	if(c != InternalCodeChar) {
 	    for(s = h + 1; *s; ++s)
 		if(isspaceByte(*s) || *s == InternalCodeChar)
@@ -1070,7 +1067,6 @@ htmlReformat(const char *buf)
 	    continue;
 	}
 
-	/* word */
 	/* It's a tag */
 	tagno = strtol(h + 1, (char **)&nh, 10);
 	c = *nh++;
@@ -1121,6 +1117,48 @@ htmlReformat(const char *buf)
 }				/* htmlReformat */
 
 
+/*********************************************************************
+Convert a 31 bit unicode character into utf8.
+*********************************************************************/
+
+static int
+uni2utf8(unsigned int unichar, unsigned char *outbuf)
+{
+    int n = 0;
+    if((unichar >= 0) && (unichar <= 0x7f)) {
+	outbuf[n++] = (unsigned char)(unichar & 0x7f);
+    } else if((unichar >= 0x80) && (unichar <= 0x7ff)) {
+	outbuf[n++] = 0xc0 | (unsigned char)((unichar >> 6) & 0x1f);
+	outbuf[n++] = 0x80 | (unsigned char)(unichar & 0x3f);
+    } else if((unichar >= 0x800) && (unichar <= 0xffff)) {
+	outbuf[n++] = 0xe0 | (unsigned char)((unichar >> 12) & 0xf);
+	outbuf[n++] = 0x80 | (unsigned char)((unichar >> 6) & 0x3f);
+	outbuf[n++] = 0x80 | (unsigned char)(unichar & 0x3f);
+    } else if((unichar >= 0x10000) && (unichar <= 0x1fffff)) {
+	outbuf[n++] = 0xf0 | (unsigned char)((unichar >> 18) & 7);
+	outbuf[n++] = 0x80 | (unsigned char)((unichar >> 12) & 0x3f);
+	outbuf[n++] = 0x80 | (unsigned char)((unichar >> 6) & 0x3f);
+	outbuf[n++] = 0x80 | (unsigned char)(unichar & 0x3f);
+    } else if((unichar >= 0x200000) && (unichar <= 0x3ffffff)) {
+	outbuf[n++] = 0xf8 | ((unichar >> 24) & 3);
+	outbuf[n++] = 0x80 | (unsigned char)((unichar >> 18) & 0x3f);
+	outbuf[n++] = 0x80 | (unsigned char)((unichar >> 12) & 0x3f);
+	outbuf[n++] = 0x80 | (unsigned char)((unichar >> 6) & 0x3f);
+	outbuf[n++] = 0x80 | (unsigned char)(unichar & 0x3f);
+    } else if((unichar >= 0x4000000) && (unichar <= 0x7fffffff)) {
+	outbuf[n++] = 0xfc | (unsigned char)((unichar >> 30) & 1);
+	outbuf[n++] = 0x80 | (unsigned char)((unichar >> 24) & 0x3f);
+	outbuf[n++] = 0x80 | (unsigned char)((unichar >> 18) & 0x3f);
+	outbuf[n++] = 0x80 | (unsigned char)((unichar >> 12) & 0x3f);
+	outbuf[n++] = 0x80 | (unsigned char)((unichar >> 6) & 0x3f);
+	outbuf[n++] = 0x80 | (unsigned char)(unichar & 0x3f);
+    } else
+	n = -1;			/* character in range 0x80000000 - 0xffffffff ... invalid? */
+
+    return n;
+}				/* uni2utf8 */
+
+
 /*********************************************************************
 And-convert the string; you know, &nbsp; &lt; etc.
 This is the routine that makes it possible for me to read, and write,
@@ -1180,6 +1218,7 @@ andTranslate(const char *s, bool invisible)
 	"half\0\xbd",
 	"frac34\0\xbe",
 	"iquest\0\xbf",
+	"rsaquo\0*",
 	"Agrave\0\xc0",
 	"Aacute\0\xc1",
 	"Acirc\0\xc2",
@@ -1382,8 +1421,7 @@ andTranslate(const char *s, bool invisible)
 		continue;
 	    }			/* tag */
 	}
-	/* code */
-	j = 1;
+
 	if(c != '&')
 	    goto putc;
 
@@ -1413,10 +1451,10 @@ andTranslate(const char *s, bool invisible)
 	    if(!r[1]) {		/* replace with a single character */
 		c = *r;
 		if(c & 0x80 && cons_utf8) {
-		    static char utfbuf[4];
-		    utfbuf[0] = (0xc0 | ((uchar) c >> 6));
-		    utfbuf[1] = (0x80 | (c & 0x3f));
-		    utfbuf[2] = 0;
+		    static char utfbuf[8];
+		    n = (uchar) c;
+		  n_utf8:
+		    uni2utf8(n, utfbuf);
 		    r = utfbuf;
 		    goto putw;
 		}
@@ -1439,14 +1477,23 @@ andTranslate(const char *s, bool invisible)
 	    stringAndString(&new, &l, r);
 	    continue;
 	}
-	/* match */
+
 	if(andbuf[0] != '#')
 	    goto putc;
-	n = stringIsNum(andbuf + 1);
+	if(andbuf[1] == 'x' || andbuf[1] == 'X') {
+	    n = strtol(andbuf + 2, 0, 16);
+	} else
+	    n = stringIsNum(andbuf + 1);
 	if(n < 0)
 	    goto putc;
-	if(n > 255)
+	if(n > 0x7f && cons_utf8) {
+	    s += j;
+	    goto n_utf8;
+	}
+	if(n > 0xff)
 	    goto putc;
+/* This line assumes iso8859-1; if you're anything else, you're screwed! */
+/* I need to fix this some day, but most everybody is utf8 anyways. */
 	c = n;
 /* don't allow nulls */
 	if(c == 0)
@@ -1456,7 +1503,6 @@ andTranslate(const char *s, bool invisible)
 	if(c == InternalCodeChar)
 	    c = ' ';
 	s += j - 1;
-	j = 1;
 
       putc:
 	if(isalnumByte(c)) {
@@ -1673,6 +1719,42 @@ In each case a new string is allocated.
 Don't forget to free it when you're done.
 *********************************************************************/
 
+/* only 8859-1 and 8859-2 so far */
+static const int iso_unicodes[2][128] = {
+    {0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8a, 0x8b,
+	   0x8c, 0x8d, 0x8e, 0x8f,
+	   0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a,
+	   0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
+	   0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9, 0xaa,
+	   0xab, 0xac, 0xad, 0xae, 0xaf,
+	   0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba,
+	   0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
+	   0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca,
+	   0xcb, 0xcc, 0xcd, 0xce, 0xcf,
+	   0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda,
+	   0xdb, 0xdc, 0xdd, 0xde, 0xdf,
+	   0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,
+	   0xeb, 0xec, 0xed, 0xee, 0xef,
+	   0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa,
+       0xfb, 0xfc, 0xfd, 0xfe, 0xff},
+    {0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8a, 0x8b,
+	   0x8c, 0x8d, 0x8e, 0x8f,
+	   0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a,
+	   0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
+	   0xa0, 0x104, 0x2d8, 0x141, 0xa4, 0x13d, 0x15a, 0xa7, 0xa8, 0x160,
+	   0x15e, 0x164, 0x179, 0xad, 0x17d, 0x17b,
+	   0xb0, 0x105, 0x2db, 0x142, 0xb4, 0x13e, 0x15b, 0x2c7, 0xb8, 0x161,
+	   0x15f, 0x165, 0x17a, 0x2dd, 0x17e, 0x17c,
+	   0x154, 0xc1, 0xc2, 0x102, 0xc4, 0x139, 0x106, 0xc7, 0x10c, 0xc9,
+	   0x118, 0xcb, 0x11a, 0xcd, 0xce, 0x10e,
+	   0x110, 0x143, 0x147, 0xd3, 0xd4, 0x150, 0xd6, 0xd7, 0x158, 0x16e,
+	   0xda, 0x170, 0xdc, 0xdd, 0x162, 0xdf,
+	   0x155, 0xe1, 0xe2, 0x103, 0xe4, 0x13a, 0x107, 0xe7, 0x10d, 0xe9,
+	   0x119, 0xeb, 0x11b, 0xed, 0xee, 0x10f,
+	   0x111, 0x144, 0x148, 0xf3, 0xf4, 0x151, 0xf6, 0xf7, 0x159, 0x16f,
+       0xfa, 0x171, 0xfc, 0xfd, 0x163, 0x2d9},
+};
+
 void
 iso2utf(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p)
 {
@@ -1680,6 +1762,8 @@ iso2utf(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p)
     int nacount = 0;
     char c;
     char *outbuf;
+    const int *isoarray = iso_unicodes[type8859 - 1];
+    int ucode;
 
     if(!inbuflen) {
 	*outbuf_p = EMPTYSTRING;
@@ -1701,8 +1785,9 @@ iso2utf(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p)
 	    outbuf[j++] = c;
 	    continue;
 	}
-	outbuf[j++] = ((uchar) c >> 6) | 0xc0;
-	outbuf[j++] = (c & 0x3f) | 0x80;
+	ucode = isoarray[c & 0x7f];
+	outbuf[j++] = (ucode >> 6) | 0xc0;
+	outbuf[j++] = (ucode & 0x3f) | 0x80;
     }
     outbuf[j] = 0;		/* just for fun */
 
@@ -1713,9 +1798,11 @@ iso2utf(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p)
 void
 utf2iso(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p)
 {
-    int i, j;
+    int i, j, k;
     char c;
     char *outbuf;
+    const int *isoarray = iso_unicodes[type8859 - 1];
+    int ucode;
 
     if(!inbuflen) {
 	*outbuf_p = EMPTYSTRING;
@@ -1734,21 +1821,29 @@ utf2iso(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p)
 	    continue;
 	}
 
-/* Convertable into 8 bit */
-	if(((uchar) c & 0xfc) == 0xc0 && ((uchar) inbuf[i + 1] & 0xc0) == 0x80) {
-	    outbuf[j++] = ((uchar) c << 6) | (inbuf[i + 1] & 0x3f);
-	    ++i;
-	    continue;
+/* Convertable into 11 bit */
+	if(((uchar) c & 0xe0) == 0xc0 && ((uchar) inbuf[i + 1] & 0xc0) == 0x80) {
+	    ucode = c & 0x1f;
+	    ucode <<= 6;
+	    ucode |= (inbuf[i + 1] & 0x3f);
+	    for(k = 0; k < 128; ++k)
+		if(isoarray[k] == ucode)
+		    break;
+	    if(k < 128) {
+		outbuf[j++] = k | 0x80;
+		++i;
+		continue;
+	    }
 	}
 
-/* Higher unicodes, more than 2 bytes, are converted into 0x80 */
+/* unicodes not found in our iso class are converted into stars */
 	c <<= 1;
 	++i;
 	for(++i; c < 0; ++i, c <<= 1) {
 	    if(((uchar) outbuf[i] & 0xc0) != 0x80)
 		break;
 	}
-	outbuf[j++] = 0x80;
+	outbuf[j++] = '*';
 	--i;
     }
     outbuf[j] = 0;		/* just for fun */
