@@ -190,7 +190,7 @@ static const struct tagInfo elements[] = {
     {"DL", "a definition list", TAGACT_NOP, 3, 5, 1},
     {"HR", "a horizontal line", TAGACT_HR, 0, 5, 5},
     {"FORM", "a form", TAGACT_FORM, 1, 0, 1},
-    {"BUTTON", "a button", TAGACT_BUTTON, 0, 0, 5},
+    {"BUTTON", "a button", TAGACT_INPUT, 0, 0, 5},
     {"FRAME", "a frame", TAGACT_FRAME, 0, 2, 5},
     {"IFRAME", "a frame", TAGACT_FRAME, 0, 2, 5},
     {"MAP", "an image map", TAGACT_MAP, 0, 2, 5},
@@ -676,8 +676,12 @@ htmlInput(void)
 	    browseError(MSG_InputType, s);
 	    n = INP_TEXT;
 	}
+    } else if(stringEqual(topTag->info->name, "BUTTON")) {
+	n = INP_BUTTON;
     }
+
     topTag->itype = n;
+
     if(htmlAttrPresent(topAttrib, "readonly"))
 	topTag->rdonly = true;
     s = htmlAttrVal(topAttrib, "maxlength");
@@ -992,6 +996,7 @@ findOpenTag(const char *name)
     bool closing = topTag->slash;
     bool match;
     const char *desc = topTag->info->desc;
+
     foreachback(t, htmlStack) {
 	if(t == topTag)
 	    continue;		/* last one doesn't count */
@@ -1257,16 +1262,13 @@ encodeTags(char *html)
 	topAttrib = t->attrib = j ? pullString(attrib, j) : EMPTYSTRING;
 
 	open = 0;
-	if(ti->nest) {
+	if(ti->nest && slash) {
 	    open = findOpenTag(ti->name);
-	    if(slash) {
-		if(!open)
-		    continue;	/* unbalanced </ul> means nothing */
-		open->balanced = t->balanced = true;
-		if(open->jv)
-		    establish_innerHTML(open->jv, open->inner, save_h, false);
-	    } else
-		open = 0;
+	    if(!open)
+		continue;	/* unbalanced </ul> means nothing */
+	    open->balanced = t->balanced = true;
+	    if(open->jv)
+		establish_innerHTML(open->jv, open->inner, save_h, false);
 	}
 
 	if(slash && ti->bits & TAG_NOSLASH)
@@ -2277,8 +2279,6 @@ lineHasTag(const char *p, const char *s)
 	if(c != InternalCodeChar)
 	    continue;
 	j = strtol(p, (char **)&p, 10);
-	if(*p != '*')
-	    continue;
 	t = list[j];
 	if(t->action != TAGACT_A)
 	    continue;
@@ -2882,7 +2882,7 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
 			goto fail;
 		    continue;
 		}		/* attach */
-		if(!unfoldBuffer(cx, textAreaDosNewlines, &cxbuf, &cxlen))
+		if(!unfoldBuffer(cx, true, &cxbuf, &cxlen))
 		    goto fail;
 		for(j = 0; j < cxlen; ++j)
 		    if(cxbuf[j] == 0) {
