@@ -12,7 +12,8 @@
 /* Define the globals that are declared in eb.h. */
 /* See eb.h for descriptive comments. */
 
-const char *version = "3.3.4";
+
+const char *version = "3.4.1";
 char *userAgents[10], *currentAgent, *currentReferrer;
 const char eol[] = "\r\n";
 char EMPTYSTRING[] = "";
@@ -29,13 +30,13 @@ struct MIMETYPE mimetypes[MAXMIME];
 static int maxTables;
 static struct DBTABLE dbtables[MAXDBT];
 char *dbarea, *dblogin, *dbpw;	/* to log into the database */
+bool fetchBlobColumns;
 char *proxy_host;
 int proxy_port;
 bool caseInsensitive, searchStringsAll;
-bool textAreaDosNewlines = true, undoable;
+bool undoable;
 bool allowRedirection = true, allowJS = true, sendReferrer = false;
 bool binaryDetect = true;
-char ftpMode;
 bool showHiddenFiles, helpMessagesOn;
 uchar dirWrite, endMarks;
 int context = 1;
@@ -166,7 +167,7 @@ readConfigFile(void)
 	"tname", "tshort", "cols", "keycol",
 	"adbook", "ipblack", "maildir", "agent",
 	"jar", "nojs", "spamcan",
-	"webtimer", "mailtimer", "certfile", "database", "proxy",
+	"webtimer", "mailtimer", "certfile", "datasource", "proxy",
 	0
     };
 
@@ -598,17 +599,7 @@ readConfigFile(void)
 	    continue;
 
 	case 27:
-	    dbarea = v;
-	    v = strchr(v, ',');
-	    if(!v)
-		continue;
-	    *v++ = 0;
-	    dblogin = v;
-	    v = strchr(v, ',');
-	    if(!v)
-		continue;
-	    *v++ = 0;
-	    dbpw = v;
+	    setDataSource(v);
 	    continue;
 
 	case 28:		/* proxy */
@@ -954,6 +945,27 @@ eeCheck(void)
 	ebClose(1);
 }
 
+void
+setDataSource(char *v)
+{
+    dbarea = dblogin = dbpw = 0;
+    if(!v)
+	return;
+    if(!*v)
+	return;
+    dbarea = v;
+    v = strchr(v, ',');
+    if(!v)
+	return;
+    *v++ = 0;
+    dblogin = v;
+    v = strchr(v, ',');
+    if(!v)
+	return;
+    *v++ = 0;
+    dbpw = v;
+}				/* setDataSource */
+
 /* I'm not going to expand wild card arguments here.
  * I don't need to on Unix, and on Windows there is a
  * setargv.obj, or something like that, that performs the expansion.
@@ -995,7 +1007,7 @@ main(int argc, char **argv)
     sprintf(configFile, "%s/.ebrc", home);
 
     recycleBin = allocMem(strlen(home) + 8);
-    sprintf(recycleBin, "%s/%s", home, TRASHDIR);
+    sprintf(recycleBin, "%s/.Trash", home);
     edbrowseTempFile = allocMem(strlen(recycleBin) + 8 + 6);
 /* The extra 6 is for the suffix */
     sprintf(edbrowseTempFile, "%s/eb_tmp", recycleBin);
@@ -1076,6 +1088,9 @@ main(int argc, char **argv)
 	i_printfExit(MSG_Usage);
     }				/* options */
 
+    if(debugLevel >= 4)
+	curl_easy_setopt(curl_handle, CURLOPT_VERBOSE, 1);
+
     if(tcp_init() < 0)
 	debugPrint(4, "tcp failure, could not identify this machine");
     else
@@ -1131,6 +1146,7 @@ main(int argc, char **argv)
     }
 
     cookiesFromJar();
+    my_curl_init();
 
     signal(SIGINT, catchSig);
     siginterrupt(SIGINT, 1);
@@ -1147,8 +1163,8 @@ main(int argc, char **argv)
 	if(cx == 1)
 	    runEbFunction("init");
 	changeFileName = 0;
-	fetchHistory(0, 0);	/* reset history */
 	cw->fileName = cloneString(file);
+	cw->firstURL = cloneString(file);
 	if(isSQL(file))
 	    cw->sqlMode = true;
 	rc = readFile(file, "");
@@ -1162,13 +1178,11 @@ main(int argc, char **argv)
 	    cw->fileName = changeFileName;
 	    changeFileName = 0;
 	}
-	if(cw->fileName && memEqualCI(cw->fileName, "ftp://", 6)) {
-	    nzFree(cw->fileName);
-	    cw->fileName = 0;
-	}
+
 	cw->firstOpMode = cw->changeMode = false;
 /* Browse the text if it's a url */
-	if(rc && !(cw->binMode | cw->dirMode) && cw->dol && isURL(cw->fileName)) {
+	if(rc && !(cw->binMode | cw->dirMode) && cw->dol &&
+	   isBrowseableURL(cw->fileName)) {
 	    if(runCommand("b"))
 		debugPrint(1, "%d", fileSize);
 	    else
