@@ -12,6 +12,16 @@ The array of messages is selected by the language.
 
 
 
+
+
+
+
+
+
+
+
+
+
 enum {
     MSG_EOF,
     MSG_NoFile,
@@ -325,7 +335,7 @@ enum {
     MSG_FTPCDTime,
     MSG_FTPBadURL,
     MSG_FTPUsage,
-    MSG_FTPConfig,
+    MSG_FTPPassword,
     MSG_FTPLibrary,
     MSG_FTPSession,
     MSG_FTPUnexpected,
@@ -635,4 +645,34 @@ enum {
     MSG_IUConvertOn,
     MSG_CopyMoveDir,
     MSG_NoDirDelete,
+    MSG_LibcurlNoInit,
+    MSG_FTPEmptyDir,
+    MSG_ReAscii,
+    MSG_ReUtf8,
+    MSG_BadUtf8String,
+    MSG_DBManyKeyCol,
+    MSG_DBNextSerial,
+    MSG_ShowTables,
+    MSG_DBNoSource,
+    MSG_DBConnecting,
+    MSG_Row,
+    MSG_Rows,
+    MSG_Selected,
+    MSG_Inserted,
+    MSG_Updated,
+    MSG_Deleted,
+    MSG_ProcExec,
+    MSG_Table,
+    MSG_UnterminatedSelect,
+    MSG_DBNoWhere,
+    MSG_RemoteAccessDenied,
+    MSG_AuthRequired,
+    MSG_NoMem,
+    MSG_CurlVersion,
+    MSG_Timeout,
+    MSG_CurlSendData,
+    MSG_CurlCatchAll,
+    MSG_Fkeys,
+    MSG_FetchBlobOff,
+    MSG_FetchBlobOn,
 };
