@@ -241,6 +241,7 @@ serverPutGet(const char *line, bool secure)
 	return false;
     if(!serverGetLine(secure))
 	return false;
+    return true;
 }				/* serverPutGet */
 
 void
@@ -400,6 +401,28 @@ isoEncode(char *start, char *end)
     return t;
 }				/* isoEncode */
 
+/*********************************************************************
+Return a string that defines the charset of the outgoing mail.
+This just looks at your language setting - reaaly dumb.
+It should interrogate each file/attachment.
+Well, this will get us started.
+*********************************************************************/
+
+static char *
+charsetString(const char *ct, const char *ce)
+{
+    static char buf[24];
+    buf[0] = 0;
+    if(!stringEqual(ce, "7bit") &&
+       (stringEqual(ct, "text/plain") || stringEqual(ct, "text/html"))) {
+	if(cons_utf8)
+	    strcpy(buf, "; charset=utf-8");
+	else
+	    sprintf(buf, "; charset=iso-8859-%d", type8859);
+    }
+    return buf;
+}				/* charsetString */
+
 /* Read a file into memory, mime encode it,
  * and return the type of encoding and the encoded data.
  * Last three parameters are result parameters.
@@ -540,7 +563,6 @@ encodeAttachment(const char *file, int ismail, bool webform,
 	}			/* .signature */
     }
 
-    /* primary email message */
     /* Infer content type from the filename */
     ct = 0;
     s = strrchr(file, '.');
@@ -594,98 +616,102 @@ encodeAttachment(const char *file, int ismail, bool webform,
        file, buflen, nacount, nullcount, longline);
     nacount += nullcount;
 
-/* Criteria for base64 encode.
- * But, some web servers won't take qp encode, so if this is a web form,
- * and we would normally qp encode, do base 64. */
+/* Set the type of attachment */
+    if(buflen > 20 && nacount * 5 > buflen) {
+	if(!ct)
+	    ct = "application/octet-stream";	/* default type for binary */
+    }
+    if(!ct)
+	ct = "text/plain";
 
-    if(buflen > 20 && nacount * 5 > buflen ||
-       webform && (nacount * 20 > buflen || nullcount || longline)) {
+/* Criteria for base64 encode.
+ * files uploaded from a web form need not be encoded, unless they contain
+ * nulls, which is a quirk of my slapped together software. */
+
+    if(!webform && (buflen > 20 && nacount * 5 > buflen) ||
+       webform && nullcount) {
 	if(ismail) {
 	    setError(MSG_MailBinary, file);
 	    goto freefail;
 	}
-
 	s = base64Encode(buf, buflen, true);
 	nzFree(buf);
 	buf = s;
-	if(!ct)
-	    ct = "application/octet-stream";	/* default type */
 	ce = "base64";
 	goto success;
     }
 
-    if(!ct)
-	ct = "text/plain";
-
+    if(!webform) {
 /* Switch to unix newlines - we'll switch back to dos later. */
-    v = buf + buflen;
-    for(s = t = buf; s < v; ++s) {
-	c = *s;
-	if(c == '\r' && s < v - 1 && s[1] == '\n')
-	    continue;
-	*t++ = c;
-    }
-    buflen = t - buf;
+	v = buf + buflen;
+	for(s = t = buf; s < v; ++s) {
+	    c = *s;
+	    if(c == '\r' && s < v - 1 && s[1] == '\n')
+		continue;
+	    *t++ = c;
+	}
+	buflen = t - buf;
 
 /* Do we need to use quoted-printable? */
 /* Perhaps this hshould read (nacount > 0) */
-    if(nacount * 20 > buflen || nullcount || longline) {
-	char *newbuf;
-	int l, colno = 0, space = 0;
+	if(nacount * 20 > buflen || nullcount || longline) {
+	    char *newbuf;
+	    int l, colno = 0, space = 0;
 
-	newbuf = initString(&l);
-	v = buf + buflen;
-	for(s = buf; s < v; ++s) {
-	    c = *s;
+	    newbuf = initString(&l);
+	    v = buf + buflen;
+	    for(s = buf; s < v; ++s) {
+		c = *s;
 /* do we have to =expand this character? */
-	    if(c < '\n' && c != '\t' ||
-	       c == '=' ||
-	       c == '\xff' ||
-	       (c == ' ' || c == '\t') && s < v - 1 && s[1] == '\n') {
-		char expand[4];
-		sprintf(expand, "=%02X", (uchar) c);
-		stringAndString(&newbuf, &l, expand);
-		colno += 3;
-	    } else {
-		stringAndChar(&newbuf, &l, c);
-		++colno;
-	    }
-	    if(c == '\n') {
-		colno = space = 0;
-		continue;
-	    }
-	    if(c == ' ' || c == '\t')
-		space = l;
-	    if(colno < 72)
-		continue;
-	    if(s == v - 1)
-		continue;
+		if(c < '\n' && c != '\t' ||
+		   c == '=' ||
+		   c == '\xff' ||
+		   (c == ' ' || c == '\t') && s < v - 1 && s[1] == '\n') {
+		    char expand[4];
+		    sprintf(expand, "=%02X", (uchar) c);
+		    stringAndString(&newbuf, &l, expand);
+		    colno += 3;
+		} else {
+		    stringAndChar(&newbuf, &l, c);
+		    ++colno;
+		}
+		if(c == '\n') {
+		    colno = space = 0;
+		    continue;
+		}
+		if(c == ' ' || c == '\t')
+		    space = l;
+		if(colno < 72)
+		    continue;
+		if(s == v - 1)
+		    continue;
 /* If newline's coming up anyways, don't force another one. */
-	    if(s[1] == '\n')
-		continue;
-	    i = l;
-	    if(!space || space == i) {
-		stringAndString(&newbuf, &l, "=\n");
-		colno = space = 0;
-		continue;
-	    }
-	    colno = i - space;
-	    stringAndString(&newbuf, &l, "**");	/* make room */
-	    while(i > space) {
-		newbuf[i + 1] = newbuf[i - 1];
-		--i;
-	    }
-	    newbuf[space] = '=';
-	    newbuf[space + 1] = '\n';
-	    space = 0;
-	}			/* loop over characters */
+		if(s[1] == '\n')
+		    continue;
+		i = l;
+		if(!space || space == i) {
+		    stringAndString(&newbuf, &l, "=\n");
+		    colno = space = 0;
+		    continue;
+		}
+		colno = i - space;
+		stringAndString(&newbuf, &l, "**");	/* make room */
+		while(i > space) {
+		    newbuf[i + 1] = newbuf[i - 1];
+		    --i;
+		}
+		newbuf[space] = '=';
+		newbuf[space + 1] = '\n';
+		space = 0;
+	    }			/* loop over characters */
 
-	nzFree(buf);
-	buf = newbuf;
-	ce = "quoted-printable";
-	goto success;
+	    nzFree(buf);
+	    buf = newbuf;
+	    ce = "quoted-printable";
+	    goto success;
+	}
     }
-    /* quoted printable */
+
     buf[buflen] = 0;
     ce = (nacount ? "8bit" : "7bit");
 
@@ -757,7 +783,7 @@ appendAttachment(const char *s, char **out, int *l)
 char *
 makeBoundary(void)
 {
-    static char boundary[60];
+    static char boundary[24];
     sprintf(boundary, "nextpart-eb-%06d", rand() % 1000000);
     return boundary;
 }				/* makeBoundary */
@@ -1036,8 +1062,8 @@ sendMail(int account, const char **recipients, const char *body,
     if(!mustmime) {
 /* no mime components required, we can just send the mail. */
 	sprintf(serverLine,
-	   "Content-type: %s%sContent-Transfer-Encoding: %s%s%s", ct, eol, ce,
-	   eol, eol);
+	   "Content-Type: %s%s%sContent-Transfer-Encoding: %s%s%s", ct,
+	   charsetString(ct, ce), eol, ce, eol, eol);
 	stringAndString(&out, &j, serverLine);
     } else {
 	sprintf(serverLine,
@@ -1049,8 +1075,8 @@ sendMail(int account, const char **recipients, const char *body,
 this format, some or all of this message may not be legible.\r\n\r\n--");
 	stringAndString(&out, &j, boundary);
 	sprintf(serverLine,
-	   "%sContent-type: %s%sContent-Transfer-Encoding: %s%s%s", eol, ct,
-	   eol, ce, eol, eol);
+	   "%sContent-Type: %s%s%sContent-Transfer-Encoding: %s%s%s", eol, ct,
+	   charsetString(ct, ce), eol, ce, eol, eol);
 	stringAndString(&out, &j, serverLine);
     }
 
@@ -1063,8 +1089,8 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
 	for(i = 0; s = attachments[i]; ++i) {
 	    if(!encodeAttachment(s, 0, false, &ct, &ce, &encoded))
 		return false;
-	    sprintf(serverLine, "%s--%s%sContent-Type: %s", eol, boundary, eol,
-	       ct);
+	    sprintf(serverLine, "%s--%s%sContent-Type: %s%s", eol, boundary,
+	       eol, ct, charsetString(ct, ce));
 	    stringAndString(&out, &j, serverLine);
 /* If the filename has a quote in it, forget it. */
 /* Also, suppress filename if this is an alternate presentation. */
